//
// Active contour example
//
// Bala Amavasai (bala@amavasai.org)
// Sun Apr 29 22:57:26 BST 2001
//

#include <iostream>
#include "active_contour.h"
#include "image.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "mimasexception.h"

using namespace std; 
using namespace mimas;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    activeContour<int> ac;
    image<int> inputimage, initialimage;

    x11_display disp;
    image_mesaoutput< unsigned char > win( &disp );

    MMERROR( argc >= 6, mimasexception, ,
	     "Syntax : " << argv[0] << " <pgm file> <circleCentre x> "
	     "<circleCentre y> <circleRadius> <numPoints> "
	     "[<gray level [0, 255]>] [SMOOTH; 1 = yes, 0 = no] "
	     "[DEBUG; 1 = yes, 0 = no]" );

    int xcenter = atoi(argv[2]);
    int ycenter = atoi(argv[3]);
    int circleRadius = atoi(argv[4]);
    int numPoints = atoi(argv[5]);
    if (argc >= 7 && atoi(argv[6]) >= 0 && atoi(argv[6]) <= 255)
      ac.showPolygonOnImage_graylevel = atoi(argv[6]);
    if (argc >= 8 && atoi(argv[7]) == 1) ac.smooth = true;
#ifndef NDEBUG
    if (argc >= 9 && atoi(argv[8]) == 1) ac.debug = true;
#endif

    // read in input image
    ifstream infile1( argv[1], ios::binary );
    infile1 >> inputimage;
    ac.setPolyCircle(xcenter, ycenter, circleRadius, numPoints);

    // show initial contour on image
    ifstream infile2( argv[1], ios::binary );
    infile2 >> initialimage;
    ac.showPolygonOnImage(initialimage);
    win << initialimage << mimas::pause();

    // adapt contour - also see adaptContourList function
    ac.adaptContour(inputimage);

    // show output
    ac.showPolygonOnImage(inputimage);
    win << inputimage << mimas::pause();

    // show final coordinate set
    ac.showPolyCoords();
  } catch (exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
