#include "integral_image.h"

mimas::integral_image::integral_image(image<unsigned char> &image)
{
  integralImage.init(image.getWidth(), image.getHeight());      
  intermediateIntegralImage.init(image.getWidth(), image.getHeight());
  
  //we first do the sum along the columns
  //it_o for original image
  //it_i for intermediate 

  mimas::image<unsigned char>::iterator it_o = image.ul();
  mimas::image<local_integer_type>::iterator it_i = intermediateIntegralImage.ul();

  //The intermediate image is filled column wised:
  for (;it_o.x < image.lr().x; it_o.x++, it_i.x++)
    {
      //the first line of the intermediate image is the same
      //as the first line of the original image.
      *it_i = *it_o;
	  
      //create an iterator for each column
      mimas::image<unsigned char>::iterator it_oc(it_o);
      mimas::image<local_integer_type>::iterator it_ic(it_i);
	  
      for (it_oc.y++, it_ic.y++; it_oc.y < image.lr().y; it_oc.y++, it_ic.y++)
	{
	  *it_ic = *it_oc + it_ic[mimas::dist2D(0, -1)]; 	    
	}
    }
      
  //Now we can calculate the integral image using the intermediate image
  //suming its values along the lines
  //it_f stands for final image
      
  it_i = intermediateIntegralImage.ul();
  mimas::image<local_long_integer_type>::iterator it_f = integralImage.ul();
      
  for (;it_i.y < intermediateIntegralImage.lr().y; it_f.y++, it_i.y++)
    {
      //we copy the first column as such
      *it_f = *it_i;

      //create an iterator for each line
      mimas::image<local_integer_type>::iterator it_il(it_i);
      mimas::image<local_long_integer_type>::iterator it_fl(it_f);
	  
      for (it_fl.x++, it_il.x++; it_il.x < intermediateIntegralImage.lr().x; it_il.x++, it_fl.x++)
	{
	  *it_fl = *it_il + it_fl[mimas::dist2D(-1, 0)]; 	    
	}
    }
}
