#include <iostream>
#include <fstream>
#include "image_fileinput.h"
#include "image_fileoutput.h"
#include "image_funcs.h"
#include "integral_image.h"
#include "mimasexception.h"
#include "weak_classifier_generator.h"
#include "strong_classifier.h"

using namespace std; 
using namespace mimas;

/**
 *
 *
 *
 *
 *
 * Time-stamp: <2005-08-08 17:07:55 engmb>
 * Copyright (C) 2005 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 03/08/2005
 *
 ***********************************************************************/


int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    //  MMERROR( argc == 6, mimasexception, ,
    //	     "Syntax: " << argv[0] << " image-filename image-output-filename lowThreshold HighThreshold StdDev" );

    /*
    x11_display display;
    image_mesaoutput< unsigned char > win( &display );
    ifstream file( argv[1], ios::binary );

    file >> someimage;
    integral_image i_image(someimage);
   

    image<int> resultImage(i_image.rawData());

    normaliseIt(resultImage , 0, 255);
     
    //ofstream ofile(argv[2], ios::binary);
    //ofile<< someimage;

    win << someimage << mimas::pause();
    win << resultImage << mimas::pause();
    */
    
    vector<weak_classifier> my_classifiers;
    vector<weak_classifier>::iterator classifier_it;
    
    weak_classifier_generator classifier_generator;

    vertical_edge v_e;
    horizontal_edge h_e;
    vertical_line v_l;
    horizontal_line h_l;
    
    classifier_generator.add_feature_prototype(v_e);
    classifier_generator.add_feature_prototype(h_e);
    classifier_generator.add_feature_prototype(v_l);
    classifier_generator.add_feature_prototype(h_l);
	
    std::cerr <<"generate classifiers"<<std::endl;
    my_classifiers = classifier_generator.generate(19,19);

    std::cerr <<"end of generation, number of classifier: "<< my_classifiers.size()<<std::endl;

    //check the number of argument

    int nb_positive_example = atoi(argv[1]);
    int nb_negative_example = atoi(argv[2]);
    
    std::vector<std::string> pos_example_images;
    std::vector<std::string> neg_example_images;

    //Embed this in a class load images and do measures on it
    
    for(int i = 0; i < nb_positive_example; i++)
      {
	image<unsigned char> someimage;	
	//!!dangerous magic number	
	pos_example_images.push_back(string(argv[3+i]));
	std::ifstream file( argv[3+i], std::ios::binary );	
	file >> someimage;
       	
	integral_image i_image(someimage);
	
	std::cerr<<"Positive example, image: "<< i<<std::endl;

	image_ref *image_reference = new image_ref(std::string(argv[3+i]), .5/nb_positive_example, 1 );
	weak_classifier::add_image(image_reference);
	
	for(classifier_it = my_classifiers.begin(); classifier_it != my_classifiers.end(); classifier_it++)
	  {
	    //!! weight and status to change
	    classifier_it->measure(image_reference, i_image);
	  }
      }
    
    for(int i = 0; i < nb_negative_example; i++)
      {
	image<unsigned char> someimage;
	//!!dangerous magic number
	pos_example_images.push_back(string(argv[3+nb_positive_example + i]));
	std::ifstream file( argv[3+nb_positive_example + i], std::ios::binary );	
	file >> someimage;
	integral_image i_image(someimage);

	std::cerr<<"negative example, image "<<i<<std::endl;
	
	image_ref *image_reference = new image_ref(std::string(argv[3+nb_positive_example + i]), .5/nb_negative_example, -1);
	weak_classifier::add_image(image_reference);
	for(classifier_it = my_classifiers.begin(); classifier_it != my_classifiers.end(); classifier_it++)
	  {
	    //!! weight and status to change
	    classifier_it->measure(image_reference, i_image);
	  }
      }

    strong_classifier final_classifier;
    
    for (int i = 0; i < 200; i++)
      {
	weak_classifier::normalise_weight_and_calculate_sum();
	for(classifier_it = my_classifiers.begin(); classifier_it != my_classifiers.end(); classifier_it++)
	  {
	    classifier_it->sort_measures();
	    classifier_it->determine_threshold();
	    // std::cerr<< "classifier error is:"<< classifier_it->error_measure << std::endl;
	  }
	
	//find best classifier
	double best_error = 1.;
	std::vector<weak_classifier>::iterator best_classifier_it;
	for(classifier_it = my_classifiers.begin(); classifier_it != my_classifiers.end(); classifier_it++)
	  {
	    if(classifier_it->error_measure < best_error)
	      {
		best_error = classifier_it->error_measure;
		best_classifier_it = classifier_it;
	      }
	  }
	std::cerr<< "best classifier error is:"<< best_error << std::endl;
	best_classifier_it->feature.print();
	
	final_classifier.add(*best_classifier_it);
	weak_classifier::reweight(*best_classifier_it);
      }

    final_classifier.calculate_threshold();
    
    image<unsigned char> an_image;
    ifstream file("./test/ds9.png", ios::binary);
    file >> an_image;
    integral_image integ_image(an_image);

    std::cerr<< "Starting detection" <<std::endl;
    
    for(float scale = 1.; scale < 5 ; scale *= 1.25)
      for(int i = 0; i< integ_image.getWidth(); i++)
	for(int j = 0; j < integ_image.getHeight(); j++)
	  {
	    if(final_classifier.detect(integ_image, i, j, scale))
	      std::cerr<< "face detected at position: (x,y,scale):("<<i <<", "<<j <<", "<<scale<<")"<<std::endl; 
	  }
    
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
