//
// Example usage of edge detectors
// Bala Amavasai (bala@amavasai.org)
//
// $Header: /cvs/mimas2/examples/edgeFilters/main.cc,v 1.1.1.1 2005/08/09 15:37:41 engmb Exp $
//


#include <iostream>
#include <fstream>
#include "gauss.h"
#include "image.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "image_funcs.h"

using namespace std; 
using namespace mimas;

template< typename T >
struct _tobw: public std::unary_function< T, unsigned char > {
  unsigned char operator()( const T &x ) const
  { return x ? 255 : 0; }
};

ARRAY_OP_EX( tobw   , _tobw    )
IMAGE_OP_EX( tobw   , _tobw    )

image< bool > edgeSobelNonMaxima( image< double > &in, double threshold )
{
  image< double >
    gradX( gradSobelX( in ) ),
    gradY( gradSobelY( in ) ),
    gradSqr( sumSquares( gradX, gradY ) );
  
  return nonMaximaSuppression( gradX, gradY, gradSqr, threshold );
}

image< bool > gaussGradientNonMaxima( image< double > &in,
                                      double sigma, double threshold )
{
  image< double >
    gradX( gaussGradientX( in, sigma ) ),
    gradY( gaussGradientY( in, sigma ) ),
    gradSqr( sumSquares( gradX, gradY ) );
  
  return nonMaximaSuppression( gradX, gradY, gradSqr, threshold );
}

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    x11_display display;
    image<double> imagein;
    image_mesaoutput< unsigned char > window( &display );

    ifstream file( "../images/stills/lena-512x512.pgm", ios::binary );
    file >> imagein;

    cerr << "Sobel edge detection" << endl;
    window << normalise( edgeSobelNorm( imagein ),
                         0.0, 255.0 ) << mimas::pause();

    const double threshval = 100.0;
    cerr << "  thresholded at " << threshval << endl;
    window << bilevel( edgeSobelSqr( imagein ),
                       threshval * threshval, 0.0, 255.0 ) << mimas::pause();

    cerr << "  with non-maxima suppression" << endl;
    window << tobw( edgeSobelNonMaxima( imagein, threshval ) )
           << mimas::pause();

    const double
      sigma = 1.0,
      gaussthresh = 100.0;
    cerr << "Gauss-gradient filter with sigma " << sigma << endl;
    window << normalise( gaussGradientNorm( imagein, sigma ), 0.0, 255.0 )
           << mimas::pause();
    cerr << "  thresholded at " << gaussthresh << endl;
    window << bilevel( gaussGradientSqr( imagein, sigma ),
                       gaussthresh * gaussthresh, 0.0, 255.0 )
           << mimas::pause();
    cerr << "  with non-maxima suppression" << endl;
    window << tobw( gaussGradientNonMaxima( imagein, sigma, gaussthresh ) )
           << mimas::pause();

    cerr << "Laplacian edge detection" << endl;
    window << normalise( edgeLaplacian( imagein ), 0.0, 255.0 )
	   << mimas::pause();

    cerr << "Laplacian of Gaussian (LoG) edge detection - note to author: fix zero-crossings" << endl;
    window << normalise( edgeLoG( imagein ), 0.0, 255.0 ) << mimas::pause();
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
