#include "mimasconfig.h"
#include <GL/glut.h>
#include <image.h>
#include <image_mesaoutput.h>
#include <gnuplot.h>

using namespace mimas;
using namespace std;


image< rgba< unsigned char > > printCurve( int width, int height,
                                           double phase, double rot_z )
  throw (mimasexception)
{
  image< rgba< unsigned char > > img;
  gnuplot p( img, width, height );
  p << "set isosamples 20,20" << endl
    << "set pm3d" << endl
    << "set view ," << fmod( rot_z / M_PI * 180.0, 360.0 ) << endl
    << "set xrange[-10:20]" << endl
    << "set yrange[-10:20]" << endl
    << "set zrange[-2:2]" << endl
    << "set hid; set nokey" << endl
    << "phase = " << phase << endl
    << "len(x,y) = sqrt(x*x+y*y)" << endl
    << "wave(x,y) = cos(len(x,y)-phase)*exp(-len(x,y)*0.2)" << endl
    << "splot wave(x,y)" << endl;
  p.finish();
  return img;
}

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {

    glutInit( &argc, argv );

    x11_display disp;
    image_mesaoutput< rgba< unsigned char > > win( &disp );

    const int
      width  = 640,
      height = 480;

      for (double i=0.0; i<10.0*2.0*M_PI; i+=0.1)
        win << printCurve( width, height, i, 0.1 * i + 0.25 * M_PI );

    win << mimas::pause();

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}

