#include <GL/glut.h>
#include "mesa_offscreen.h"
#include "mesa_onscreen.h"
#include "image_mesaoutput.h"

using namespace boost;
using namespace std;
using namespace mimas;

image< rgba< unsigned char > > gaudy(void)
{
  image< rgba< unsigned char > > image; image.init( 256, 256 );
  for ( int k=0; k<=255; k++ )
    for ( int j=0; j<=255; j++ )
      image.pixel( k, j ) = rgba< unsigned char >( k, j, 255 - k );
  return image;
}

void rotatedText( mesa_base &m, int angle, const string &text ) {
  m.rectangle( 0, 0, 255, 255 );
  
  glPushMatrix();
  glTranslated( 20, 200, 0 );
  glScaled( 0.1, -0.1, 1.0 );
  glRotatef( angle, 0, 0, 1 );
  m.text( text );
  glPopMatrix();
}

int main( int argc, char *argv[] )
{
  int retVal = 0;

  // Initialise GLUT.
  glutInit( &argc, argv );

  try {

    // Open X11 display.
    x11_display display;

    // Off-screen rendering on colour- and grey-images.

    // Create two image-output windows.
    image_mesaoutput< rgba< unsigned char > > window( &display );

    // Create background images.
    image< rgba< unsigned char > > image( gaudy() );

    // Do some off-screen rendering on it.
    {
      mesa_offscreen< rgba< unsigned char > > m( &display, image, false );
      m.ortho2D();
      const rgba< unsigned char > green( 0, 128, 0 );
      m.setColor( green );
      rotatedText( m, 30, "offscreen" );
    };
      
    window << image << mimas::pause();

  } catch ( exception &e ) {

    cerr << e.what() << endl;
    retVal = 1;

  };
  
  return retVal;
}
