#include <fstream>
#include <iostream>
#include <vector>
#include <string>
#include "image.h"
#include "image_fileinput.h"
#include "image_mesaoutput.h"
#include "opticflow_diff_tool.h"
#include "image_draw.h"

using namespace std;
using namespace mimas;

/**
 * Differential optic flow - see page 196-197 of Trucco and Verri
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Wed Apr 23 16:46:27 BST 2003
 * $Header: /cvs/mimas2/examples/opticflow/main.cc,v 1.1.1.1 2005/08/09 15:37:40 engmb Exp $
 */

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {
	x11_display display;
	image_mesaoutput< unsigned char > window( &display );
	opticflow_diff_tool of;
	image<double> flow;
	image<double> xflow,yflow;
	vector< image<double> > imagein( 5 );

        MMERROR( argc == 6, mimasexception, ,
                 "Syntax: " << argv[0] << " <image1> <image2> ... <image5>" );

	// Read in five images
	// note that we are computing the flow based on all five of these
	// images!
        for ( int i=1; i<argc; i++ ) {
          ifstream file( argv[i], ios::binary );
          file >> imagein[i-1];
        };

	// Find optic flow field
	of.findFlow(imagein);

	// get flows in x and y directions and store in xflow and yflow respectively
	of.getXflow(xflow);
	of.getYflow(yflow);


	// plot the flows on the centre (third) image and display
	// note the 10x multiplier is just to scale to flow so that it
	// can be seen!
	flow.init(imagein[2].getWidth(),imagein[2].getHeight());
   flow.fill(255);

	// set background to a bright version of the third image
	for (int j=0; j<flow.getHeight(); j++)
		for (int i=0; i<flow.getWidth(); i++)
			flow.setPixel(i,j,(int)(192.0*(double)imagein[2].getPixel(i,j)/255.0)+63);
			
	for (int j=0; j<flow.getHeight(); j++)
	  for (int i=0; i<flow.getWidth(); i++)
	    if ( (xflow.getPixel(i,j)!=0.0) && (yflow.getPixel(i,j)!=0.0) )
	      {
		drawThickLine(flow, i, j, i, j, 0., 3); // plot a thick dot
		drawLine(flow, i,j,i+(int)(10*xflow.getPixel(i,j)),j+(int)(10*yflow.getPixel(i,j)),0.); // draw the line
	      }

	window << flow << mimas::pause();
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}

