#ifndef MMVL_FIND_LINES_HH
#define MMVL_FIND_LINES_HH

#include "boost/multi_array.hpp"

#define NO_DISPLAY

#include <multi_array_op.h>
#include <algorithm>
#include <vector>
#include <cmath>
#include <image.h>
#include <image_draw.h>

#ifndef NDEBUG

#include <iostream>
#include <fstream>

#endif

#ifndef NO_DISPLAY

#include "image_fileinput.h"
#include "image_qtoutput.h"
#include "image_funcs.h"
#include "property_image.h"

#include <GL/glut.h>
#include "image_mesaoutput.h"
#endif

namespace mimas{
  
  typedef boost::multi_array<double,2> Array2D;

/**
Line finding using Hough Transform.

Works on sub-images thanks to image iterators 

Pixels which value is different from 0 are considered as being candidate for
belonging to a line.

This was originally implemented as a short exercice for myself to understand
the hough transform. As such the value of the code is poor however it might
allow other people to understand the hough transform with a simple implementation
as well as providing them with a starting point for further experimentations.

One of the interesting idea that can be used more generally as a philosophy for code developement
is to have a set of low level function as well as more higher functions build over these
functions.
For instance, exec() execute an algorithm that will return an image displaying the
line found using the hough transform for lines,
however the other function are let as public if someone intend to
reuse them with a different kind of result.

It is thus possible to continue improving this class adding other high-level as well
as low level functionalities. (a bit similar with a classical layer architecture)

A bit more of work is still needed
to put it in the include of mimas.
Need to be tested for color images ...

Check what can be borrowed from hough_transform
one of the first improvement is to locate localy a maximum when adjacent bins
have number of hits above the threshold

Have a look to the gandalf implementation of the fast hough transform

at the moment a sample code using this class would look like:

find_line<image<int >::iterator > h(image.ul(), image.lr() , .7, 145);    
display << h.exec();

@author: Manuel Boissenin
@date: 28 november 2004
*/

  template<typename iteratorSource>
  class find_line{

  protected:
    int nbOrientationDiv;
    double thresh;

    double maxVal;

    //image<unsigned char> &x;
    iteratorSource ul;
    iteratorSource lr;

    int w,h;
    int imageDiagonalSize;
    int center_x, center_y;
    
    
    //stand for Polar to Cartesian helper function to convert a point
    //from the parametre space to a line in the image space. Add the line
    //to the vector of line.
    void PtoC(double theta,double r)
    {
      int test = 1000;
      //calculate the intersection point between the line and its perpenducular
      //that is passing through the origin
    
      double x = r * cos (double(theta*M_PI/nbOrientationDiv));
      double y = r * sin (double(theta*M_PI/nbOrientationDiv));
      //the direction of the line is (y,-x)
      //scale and check in  the bound of image if necessary

      //we take 2 points on the line centered on intersection,separated by
      //2*test distance
      double size = sqrt(x*x + y*y);

      Point p1 = Point( int(x + y * test /size) + center_x , center_y - int(y - x * test /size) );
      Point p2 = Point( int(x - y * test /size) + center_x , center_y - int(y + x * test /size) );

      lines.push_back(Line(p1,p2));
    }

    //this function has been adapted from Parker's book
    //it is the core of the hough transform:
    //it transforms edge points into parametre space for line 
   
    void hough (void)
    {
      double conv = M_PI/nbOrientationDiv;
      //Centering the frame on the centre of the image we divide by 2 the sample space needed
      //for r as this value can be negative and its max(|r|) is divided by 2.
      //The two first loop are used all the time -> find a generic way to apply an operation to each pixel of an image.

      iteratorSource temp(ul);
      for (int i = 0; i < w; i++, temp.x++){
	iteratorSource temp2(temp);
	for (int j = 0; j < h; j++,temp2.y++)
	  if (*temp2 != 0)         //the logic that constaint less the feature input image. (compared with == 255 for instance) 
	    for (int omega = 0; omega < nbOrientationDiv; omega++) 
	      {
		int r = (int)( (i - center_x) * cos(double(omega*conv)) - (j - center_y) * sin(double(omega*conv)));
	      
		assert(imageDiagonalSize / 2 + r > 0 ||imageDiagonalSize / 2 + r <= imageDiagonalSize);
		parameterSpace[omega][imageDiagonalSize / 2 + r] += 1;
	      }
      }
    }

    void findMax(void)
    {
      maxVal  = *(std::max_element(parameterSpace.data(), parameterSpace.data()+parameterSpace.num_elements()));
#ifndef NDEBUG
      std::cerr<< "find_line: MaxValue is: " <<maxVal<<std::endl;
#endif
    }

    void findLine(void){
      //rubish code: use template programing
      for (int t=0; t<nbOrientationDiv; t++)
	for (int r=0; r< imageDiagonalSize; r++)
	  if ((parameterSpace[t][r] > maxVal * thresh))
	    {
	      //cerr<< "thresholded value is:" << parameterSpace[t][r] <<  endl;
	      //cerr<< "angle is: " << t <<endl;
	      //cerr <<"dist is  " << r -imageDiagonalSize / 2<<endl;
	      PtoC( t,r - imageDiagonalSize /2);
	    }
    }


    image<unsigned char> displayLine(void)
    {
      image<unsigned char> x;
      x.init(w,h);
      for (typename std::vector<Line>::iterator it = lines.begin(); it != lines.end();it++)
	{
	  //!! replace this function on image: should not be there.
	  mimas::drawLine(x,it->p1.x, it->p1.y, it->p2.x, it->p2.y, (unsigned char)(100));
	}


#ifndef NO_DISPLAY  
      //Amazing element-wise multplication developed by Jan.
      image<double> toDisplay( parameterSpace * ( 255.0 / maxVal ) );

      houghDisplay << toDisplay<< mimas::pause();

      houghDisplay << x << mimas::pause();
#endif
      return x;
    }


  public:
//!!I have defined many time a point and a line how to systematize to
//include in mimas/or using a standard library??
    class Point{
    public:
      int x;
      int y;
      Point(int _x = 0, int _y = 0):
	x(_x),y(_y){}
    };
  
    class Line{
    public:
      Point p1;
      Point p2;
    
      Line(Point &p1_, Point &p2_)
	:p1(p1_),p2(p2_){}
    };
      
    std::vector<Line> lines;
    Array2D parameterSpace;

    //Take an image of feature as input value
    //The thresh_factor value is multiplied with the maximum number of hit for a bin.
    //The resulting value serves as the minimum threshold value to detemine if a bin
    //represent a  line or not (see findline())
    find_line(iteratorSource ul, iteratorSource lr, double thresh_factor = .7, int orientationDivision = 180)
      :
       nbOrientationDiv(orientationDivision),   
       thresh(thresh_factor),ul(ul),lr(lr),
       w(lr.x - ul.x),h(lr.y - ul.y),
       imageDiagonalSize(int(sqrt(double(w*w+h*h)))),
       parameterSpace(boost::extents[orientationDivision][imageDiagonalSize])//size has to be checked maybe +1 would be the right measure
#ifndef NO_DISPLAY
      ,houghDisplay(*qApp)
#endif
    {
      assert(thresh < 1);
      
      center_x = w /2;
      center_y = h /2;
     
      //!!! initialisation of the parameterSpace. Is it really necessary?    
      //check with the boost documentation
      for (int r = 0; r < imageDiagonalSize; r++)
	for (int omega = 0; omega < nbOrientationDiv; omega++)
	  parameterSpace[omega][r] = 0;
    }
  
    image<unsigned char> exec(void)
    {
      hough();
      findMax();
      findLine();  
      return displayLine();
    }

#ifndef NO_DISPLAY
    mutable image_mesaoutput<unsigned char> houghDisplay;
#endif

  };
  
}

#endif

