/**********************************************************************
* Example code how to use the hypothesis filter.
*
* Like with main3DCustom code but we are using opengl to generate the
* rubiks cube.
* The tracking doesn't work because we don't filter yet the image
* to extract the edge of the rubik's cube. 
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2006-05-12 18:01:54 engmb>
* Copyright (C) 2004 by Manuel Boissenin
*
 ***********************************************************************/
#include <string>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <sstream>

#include <mesa_offscreen.h>

#include <image.h>
#include <rgba.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
//#include <image_mesaoutput.h>

#include <gauss.h>
#include <image_funcs.h>
#include <analyse_result_mean.h>
#include <hypothesis3D.h>
#include <image_loader.h>

#include "openGL_observer.h"

using namespace mimas::hf;
using namespace std;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {

    //Initialisation tracker
    glutInit(&argc, argv); 
    mimas::x11_display disp;   
    image_mesaoutput<rgba<unsigned char> > win(&disp);
    image_mesaoutput< unsigned char > show(&disp);

    hypothesis3D init(atof(argv[2]),atof(argv[3]),atof(argv[4]),atof(argv[5]),atof(argv[6]),atof(argv[7]));
  
    hypothesis3D::transX = atof(argv[8]);
    hypothesis3D::transY = atof(argv[9]);
    hypothesis3D::transZ = atof(argv[10]);
    hypothesis3D::rotX = atof(argv[11]);
    hypothesis3D::rotY = atof(argv[12]);
    hypothesis3D::rotZ = atof(argv[13]);

    image_loader<rgba<unsigned char> >::set il_param(argv[1],99,".ppm",2);;
    openGL_observer<hypothesis3D, rgba<unsigned char> >::set obs_param(&disp);
    analyse_result_mean<hypothesis3D, rgba<unsigned char> >::set analyse_param;
  
    hypothesis_filter<image_loader, openGL_observer, analyse_result_mean, hypothesis3D,
      condensation, no_compensation, mimas::rgba<unsigned char> > hf(init, atoi(argv[14]), il_param, obs_param, analyse_param);

    //to load the distance image instead of  calculating it live. 
   
    string distance_image_basename = argv[1];
    distance_image_basename += "dist";
    
    image_loader<int > distance_map
      (hf.filteredImage,
       image_loader<int>::set(distance_image_basename, 99, ".ppm", 2, 0));


    string fileOutputBaseName = "image_result";
    int imageNumber = 0;
    
    while(true)
      {

      try{hf.next();
      distance_map.next();
	}
      catch (mimasexception &e){
	std::cerr<<e.what()<<std::endl;  
	return 0;
      }

	std::ostringstream str;
	str << fileOutputBaseName << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	    << ".ppm";
	
	ofstream fileOut(str.str().c_str(), std::ios::binary); 
	
	hypothesis3D estimatedLocation;
	//filter the image and instanciate it for the observer

	//load the distance map to edges of the image.
	image<double > gauss_im = gaussGradientSqr(image<double>(hf.image) , 2.);
	show << normalise< double >( gauss_im, 0.0, 255.0 ) << mimas::pause();
	//	mod_img = image<unsigned char>(gauss_im);

	//	filter<float>::gradX( gauss_im, grad_x );
	//filter<float>::gradY( gauss_im, grad_y );
    
	//norm = sumSquares( grad_x, grad_y );
	//mod_img = image<unsigned char>( normalise< float >( norm, 0.0f, 255.0f ) );
	//hf.filteredImage = gradient(hf.image.ul(), hf.image.lr(),40);	
	win << hf.image<< mimas::pause();
	
	show << hf.filteredImage<< mimas::pause();

	hf.filter();
	win << hf.offscreenImage<<mimas::pause();
	
	estimatedLocation = hf.analyse();

	rgba<unsigned char> red(255,0,0);
	hf.draw(estimatedLocation, red);                    //from Observer	
	win<<hf.image<<mimas::pause();
#ifndef NDEBUG
	estimatedLocation.debug();
#endif
	fileOut << hf.image;
      }

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}





