#include <boost/numeric/ublas/vector.hpp>
#include <iostream>
#include <fstream>
#include "image.h"
#include "image_fileinput.h"
#include "pgh.h"
#include "pgh_2dobject.h"
#include "pgh_scene.h"

using namespace std; 
using namespace mimas;

typedef boost::numeric::ublas::vector< double > Vector;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    image<int> someimage, imagein;
    pgh_2dobject_ptr testmodel( new pgh_2dobject );
    pgh_scene	camera_scene;

 	if(argc < 3)                                     
 	{
		cout << "Syntax : " << argv[0] << " <scene image> <model image>" << endl;            
 		return 1;
 	}

   ifstream modelfile( argv[2], ios::binary );
   modelfile >> someimage;
   testmodel->createFromImage(someimage);
   camera_scene.addModel( testmodel );

	cout << "Lines extracted from the model:" << endl;
	cout << "-------------------------------" << endl;
	
    line_ptr_list::iterator	iter;
    for (iter = testmodel->getListofLinesGUI()->begin();
            iter != testmodel->getListofLinesGUI()->end(); iter++) {

        int x1,y1,x2,y2;
        line_ptr l = *iter;
        const Vector *linetips;

        linetips = &l->getStart();
        x1 = (int)(*linetips)(0);
        y1 = (int)(*linetips)(1);

        linetips = &l->getEnd();
        x2 = (int)(*linetips)(0);
        y2 = (int)(*linetips)(1);

        cout << "x1_" << x1 << "," << "y1_" << y1 << " ";
        cout << "x2_" << x2 << "," << "y2_" << y2 << endl;
    }

    cout << "--------------------------------------------------------" << endl;


    time_t starttime,endtime, histtime;
    starttime = time(NULL);
    // int object_count = 0;

    // ****************************
    // Scene Image Read from a file
    // ****************************
    ifstream imagefile( argv[1], ios::binary );
    imagefile >> imagein;

    // ********************************************************
    // Store the Scene Image in the Camera Scene Along with the
    // "pgh_2dobject" models already Stored
    // ********************************************************
    camera_scene.updateSceneLines(imagein);

    // *******************************************************************
    // Loop through all the Models in the pgh_2d Object Array and locate
    // the Model Objects in the scene
    // *******************************************************************


    histtime  = time(NULL);
#ifndef NDEBUG
    cerr << "Looking for the object ..." << endl;
#endif

    // *****************************************************************
    // Look for the pgh_2d Object Model stored in the Slot of the
    // pgh_2d Object Array viz. "pgh_object[slot]"
    // !!If the pgh_2d Object Model is already stored in the
    // !!"camera_scene" Why do we need to pass pgh_2d Object Model Again
    // ****************************************************************
    object_instance_list_ptr oil = camera_scene.findModel( testmodel );
    endtime   = time(NULL);

    // ***********************************************************
    // Loop through all Instances of this Model in the Scene Image
    // ***********************************************************
    if (oil) {

      for ( coordinate_xform_ptr_list::iterator it =
              oil->getList().begin(); it != oil->getList().end(); it++) {
        coordinate_xform_ptr x = *it;
        
        double xdisp, ydisp;
        Vector d( x->getDisplacement() );
        double angle = (double)x->getRotation();
        
        xdisp = d(0);
        ydisp = d(1);
        
        cout << "The Displacement = ";
        cout << xdisp << "," << ydisp << " and Orientation ";
        cout << angle << endl;
        
        
      }
    }

    cout << "total search time = ";
    cout << endtime-starttime << "seconds " << endl;
    cout << "( " << histtime - starttime;
    cout << " seconds to generate hists for the scene lines " << endl;
    cout << "  and " << endtime - histtime;
    cout << " seconds to compare and locate the objects. " << endl;
    cout << "--------------------------------------------------------" << endl;
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
