#include <fstream>
#include <GL/glut.h>
#include <map>
#include "image_fileinput.h"
#include "image_fileoutput.h"
#include "image_mesaoutput.h"
#include "timer.h"

using namespace boost;
using namespace std;
using namespace mimas;

typedef rgba< unsigned char > pixel_t;

int main( int argc, char *argv[] )
{
  int retVal = 0;

  try {

    // Initialise GLUT.
    glutInit( &argc, argv );

    // Check command-line.
    if ( argc < 2 ) {

      mimasexception e;
      e << "Syntax: " << argv[0] << " <image1> [image2 ...]" << endl;

      // Retrieve information about available file-formats.
      map< string, bool >
        inputFormats( image_fileinput< pixel_t >::formats() ),
        outputFormats( image_fileoutput< pixel_t >::formats() );

      e << "Input formats:";
      for ( map< string, bool >::const_iterator i =
              inputFormats.begin(); i != inputFormats.end(); i++ )
        e << ' ' << i->first;
      e << endl;

      e << "Output formats:";
      for ( map< string, bool >::const_iterator i =
              outputFormats.begin(); i != outputFormats.end(); i++ )
        e << ' ' << i->first;

      // Throw exception.
      throw e;

    };

    // Open X11 display.
    x11_display display;

    const int delay = 200;

    map< string, shared_ptr< ifstream > > stream;
    map< string, shared_ptr< image_fileinput< pixel_t > > > input;
    map< string, shared_ptr< image_mesaoutput< pixel_t > > > window;

    // Loop for command-line arguments.
    for ( int i=1; i<argc; i++ ) {

      stream[ argv[i] ] =
        shared_ptr< ifstream >( new ifstream( argv[i], ios::binary ) );

      input[ argv[i] ] =
        shared_ptr< image_fileinput< pixel_t > >
        ( new image_fileinput< pixel_t >( *stream[ argv[i] ] ) );

      window[ argv[i] ] =
        shared_ptr< image_mesaoutput< pixel_t > >
        ( new image_mesaoutput< pixel_t >( &display ) );

    };

    bool abort;
    do {

      abort = true;

      for ( int i=1; i<argc; i++ ) {

        if ( *input[ argv[i] ] ) {

          image< pixel_t > img;
          *input[ argv[i] ] >> img;
          *window[ argv[i] ] << img << mimas::delay( delay );

          abort = false;

        };

      };
      
    } while ( !abort );

    assert( window.begin() != window.end() );
    assert( window.begin()->second );
    *window.begin()->second << mimas::pause();

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  
  return retVal;
}
