#include <fstream>
#include <GL/glut.h>
#include "image_xineinput.h"
#include "image_mesaoutput.h"

using namespace std;
using namespace mimas;

// Minimal program for accessing video data using libxine.
int main( int argc, char *argv[] )
{
  int retVal = 0;

  try {

    // Note, that this is a very inefficient way to do it, if you just want
    // to display a movie. If you just want to implement a video player, you
    // should allow libxine do write directly to X11 memory!
    // See http://xinehq.de/index.php/hackersguide#AEN80 for more information.

    // Initialise GLUT.
    glutInit( &argc, argv );

    MMERROR( argc == 2, mimasexception, ,
             "Syntax: " << argv[0] << " <media resource location>" );

    // Open X11 display.
    x11_display display;

    // Open video source.
    image_xineinput< rgba< unsigned char > > input( argv[1], "auto", "auto" );

    // Open window.
    image_mesaoutput< rgba< unsigned char > > window( &display );

    // Allocate image.
    image< rgba< unsigned char > > img;

    // Loop for images.
    do {
      
      // Grab image.
      input >> img;

      // Display image.
      window << img << mimas::delay( 100 );
      
    } while ( input );

  } catch ( exception &e ) {

    cerr << e.what() << endl;
    retVal = 1;

  };
  
  return retVal;
}
