#ifndef MMVL_ANALYSE_RESULT_BLOB_HH
#define MMVL_ANALYSE_RESULT_BLOB_HH

#define MAX(a,b) (a)>(b)?(a):(b)  
#define ABS(x) (x)>=0?(x):-(x)

#include <cassert>
#include <cmath>
#include <iostream>
#include <vector>
#include <image.h>

namespace mimas{
  namespace hf{
    
    /**@brief An analyse result policy for the hypothesis filter
     *
     *Hypothesis_T has to implement float distance(Hypothesis_T)
     *and Hypothesis_T mean(Hypothesis_T) for this policy to be implemented.
     *
     *Calculate the average position of hypotheses located in the neighborhood of 
     *an hypothesis that has a high measure. The average is weighted by the 
     *hypothesis measure. The result is a set of positions that may be considered
     *as the modes of the underlying probability density function of presence of 
     *the object.
     *
     *@author Manuel Boissenin
     */
    template<typename Hypothesis_T, typename Pixel_T>
    class analyse_result_blob{

    public:
     /**@brief to store the parameters of the constructor of analyse_result_blob
     *
     *The reason we need a class to do that is that different analyse policy 
     *need a different number of parameters. However it is the hypothesis_filter constructor, which have a fix
     *number of parametres, that instanciate the analyse policy constructor. Using an intermediate class we are
     *able to pass a variable number of parametres thanks to a fix number of variables.     
     *
     *@author Manuel Boissenin
     */
      class set
      {
      public:
	float DISTANCE;
	float THRESHOLD_TO_BE_CENTER;
	float MERGE_DISTANCE;
	
	set(double threshold_to_be_center, float peak_distance, float merge_distance)
	  :DISTANCE(peak_distance), THRESHOLD_TO_BE_CENTER(threshold_to_be_center),
	   MERGE_DISTANCE(merge_distance)
	{}
      };
      
      analyse_result_blob(std::vector<Hypothesis_T> &hypotheses, image<Pixel_T> &image, set analyse_param)
	: peak_distance(analyse_param.DISTANCE), weight_threshold(analyse_param.THRESHOLD_TO_BE_CENTER),
	  merge_distance(analyse_param.MERGE_DISTANCE), 
	  hypotheses_AR(hypotheses), image_AR(image)
      {}
      
      std::vector<Hypothesis_T> analyse(void);
      
    private:

      float peak_distance;
      float weight_threshold;
      float merge_distance;

      std::vector< std::vector<Hypothesis_T> > peaks;
      std::vector<Hypothesis_T> blobs;
      
      std::vector<Hypothesis_T> &hypotheses_AR;
      image<Pixel_T> &image_AR;  
    };

#include "analyse_result_blob.tcc"    
  }
}
#endif
