/**********************************************************************
 * Description: 
 * Specific to the 2D tracking. Consider it as an instance how to use
 * the particle filter.
 *
 *
 * @author: Manuel Boissenin
 * @date: 20/05/2004
 *
 * Time-stamp: <2005-04-29 16:57:05 engmb>
 * Copyright (C) 2004 by Manuel Boissenin
 *
 ***********************************************************************/

#ifndef MMVL_PF_ANALYSE_RESULT_MEAN_HH_INCLUDED
#define MMVL_PF_ANALYSE_RESULT_MEAN_HH_INCLUDED

#include <iostream>
#include <cassert>
#include <cmath>
#include <cstdlib>
#include <vector>
#include <limits>

#include "hypothesis_filter.h"

namespace mimas{
  namespace hf{

    /**@ingroup pf
       @brief An analyse result policy for the hypothesis filter
    
       Calculate the average position of the hypotheses weighted by
       their measure. Can work with hypothesis2D and hypothesis3D
    */

    template <typename Hypothesis_T, typename Pixel_T>
    class analyse_result_mean{
    public:
     /**@brief to store the parameters of the constructor of analyse_result_blob
     *
     *The reason we need a class to do that is that different analyse policy 
     *need a different number of parameters. However it is the hypothesis_filter constructor, which have a fix
     *number of parametres, that instanciate the analyse policy constructor. Using an intermediate class we are
     *able to pass a variable number of parametres thanks to a fix number of variables.     
     *
     *@author Manuel Boissenin
     */      
      class set{
      public:
	set(void){}
      };
      

      /**The image is passed as a parameter even if it is not used
	 as another AnalyseResult policy might need more information from the
	 image.
      */
      analyse_result_mean(std::vector<Hypothesis_T> &hypotheses, image<Pixel_T> &image, set nothing)
	:hypotheses_AR(hypotheses), image_AR(image)
	{}
      
      Hypothesis_T analyse(void);

    private:
      const std::vector<Hypothesis_T> &hypotheses_AR;       
      image<Pixel_T> &image_AR;
    };
    
#include "analyse_result_mean.tcc"

  }
}

#endif
