//
// Background subtraction based on template
//
// Bala Amavasai (bala@amavasai.org)
// Mon Sep 24 11:29:02 BST 2001  
//
// $Header: /cvs/mimas2/include/bg_subtract.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
//
 
#ifndef BG_SUBTRACT_H
#define BG_SUBTRACT_H
 
/* #pragma interface  */

#include <fstream>
#include <iostream>
#include <list>
#include <cmath>
#include <values.h>
#include "image.h"
#include "image_fileoutput.h"

namespace mimas {
	
template<typename T>
class bg_subtract
{
    private:
	image<T>	bgtemplate;

    protected:
	double averageVariation;
	bool removeMeans;

    public:
	void createTemplate(image<T> &imagein, int xroi1, int yroi1, int xroi2, int yroi2);
	void createTemplate(image<T> &imagein);
	void removeBackground(image<T> &imagein);
	
    bg_subtract()
    {
	averageVariation=20.0;
	removeMeans=false;
//	averageVariation=0.5;
//	removeMeans=true;
    }

};

template <typename T>
void bg_subtract<T>::createTemplate(image<T> &imagein, int xroi1, int yroi1, int xroi2, int yroi2)
{
    int swap;
    
    if (xroi1>xroi2)
    {
	swap=xroi2; 
	xroi2=xroi1; 
	xroi1=swap;
    }
    if (yroi1>yroi2)
    {
	swap=yroi2; 
	yroi2=yroi1; 
	yroi1=swap;
    }

    xroi1=xroi1<0?0:xroi1;
    yroi1=yroi1<0?0:yroi1;
    
    xroi2=xroi2>=imagein.getWidth()?imagein.getWidth()-1:xroi2;
    yroi2=yroi2>=imagein.getHeight()?imagein.getHeight()-1:yroi2;
    //cerr << xroi1 << " " << yroi1 << " " << xroi2 << " " << yroi2 << std::endl;
    
    bgtemplate.init(xroi2-xroi1+1, yroi2-yroi1+1);
	
    int maxval=INT_MIN, minval=INT_MAX;
    for (int y=yroi1, j=0; y<=yroi2; y++, j++)
	for (int x=xroi1, i=0; x<=xroi2; x++, i++)
	{
       const T &value = imagein.pixel(x,y);
	    if ( value>maxval ) maxval = value;
	    if ( value<minval ) minval = value;
       bgtemplate.pixel(i,j) = value;
	}

    if ((removeMeans) && (maxval>minval))
    {	
        for (int y=yroi1, j=0; y<=yroi2; y++, j++)
	    for (int x=xroi1, i=0; x<=xroi2; x++, i++)
         bgtemplate.pixel(i,j) = (T)((double)(bgtemplate.pixel(i,j)-minval)/(double)(maxval-minval));
    }
   
    std::ofstream outfile( "template.pgm", std::ios::binary );
    outfile << bgtemplate;
}

template <typename T>
void bg_subtract<T>::createTemplate(image<T> &imagein)
{
    createTemplate(imagein, 0, 0, imagein.getWidth()-1, imagein.getHeight()-1);
}

template <typename T>
void bg_subtract<T>::removeBackground(image<T> &imagein)
{
    image<T> tempimage;
    double sumerr, err;
    int numpix; 
    
    tempimage.init(imagein.getWidth(),imagein.getHeight());
    
    for (int y=0; y<imagein.getHeight(); y++)
        for (int x=0; x<imagein.getWidth(); x++)
	{
	    sumerr=0.0;
	    numpix=0;
	    
	    int maxval=INT_MIN, minval=INT_MAX;

	    if (removeMeans)
	    {
	      for (int yt=-bgtemplate.getHeight()/2; yt<=bgtemplate.getHeight()/2; yt++)
		for (int xt=-bgtemplate.getWidth()/2; xt<=bgtemplate.getWidth()/2; xt++)
		{
		  if ((xt+x<0) || (xt+x>=imagein.getWidth())) continue;
		  if ((yt+y<0) || (yt+y>=imagein.getHeight())) continue;
		  if (imagein.pixel(xt+x,yt+y)>maxval) maxval=imagein.pixel(xt+x,yt+y);
		  if (imagein.pixel(xt+x,yt+y)<minval) minval=imagein.pixel(xt+x,yt+y);

		}		
	    } 
		    
	    for (int yt=-bgtemplate.getHeight()/2; yt<=bgtemplate.getHeight()/2; yt++)
	      for (int xt=-bgtemplate.getWidth()/2; xt<=bgtemplate.getWidth()/2; xt++)
	      {
		  if ((xt+x<0) || (xt+x>=imagein.getWidth())) continue;
		  if ((yt+y<0) || (yt+y>=imagein.getHeight())) continue;
		  numpix++;

		  if ((removeMeans) && (maxval>minval))
		  {
		    err=fabs(((double)(imagein.pixel(xt+x,yt+y)-minval)/(double)(maxval-minval))-
			  (double)(bgtemplate.pixel(xt+bgtemplate.getWidth()/2,yt+bgtemplate.getHeight()/2)));
		  }
		  else
		  {
		    err=(double)abs(imagein.pixel(xt+x,yt+y)-
			bgtemplate.pixel(xt+bgtemplate.getWidth()/2,yt+bgtemplate.getHeight()/2));
		  }
		  
		  sumerr=sumerr+(double)err;
		
	      }
	    
	    if ((sumerr/(double)numpix)<=averageVariation)
         tempimage.pixel(x,y)=1;
	    else
         tempimage.pixel(x,y)=0;
	    
	}
    
    for (int y=0; y<imagein.getHeight(); y++)
        for (int x=0; x<imagein.getWidth(); x++)
	    if ((tempimage.pixel(x,y))==1)
		for (int yt=-bgtemplate.getHeight()/2; yt<=bgtemplate.getHeight()/2; yt++)
		    for (int xt=-bgtemplate.getWidth()/2; xt<=bgtemplate.getWidth()/2; xt++)
		    {
			if ((xt+x<0) || (xt+x>=imagein.getWidth())) continue;
			if ((yt+y<0) || (yt+y>=imagein.getHeight())) continue;
	
			imagein.pixel(x+xt,y+yt)=0;
		    }
}

}

#endif
