//
// corner (element) property
// stumeikle Sun May 12 20:31:50 2002
//
//

#ifndef CORNER_H
#define CORNER_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include "angle.h"
#include "prop.h"

namespace mimas {

/**
 *
 * Another image property.. this is a small structure used to contain a small amount
 * of data associated with a found corner, and is used to render the corner too.
 *
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Sun May 12 20:31:50 2002
 * $Header: /cvs/mimas2/include/corner.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */
class corner : public prop
{
   public:
   typedef boost::numeric::ublas::vector< double > Vector;
    private:
   Vector centre;		///< position wrt the image plane
	double		strength;	///< contrast/strength of the corner
	angle 	orientation;	///< orientation of the corner

    public:
	void 	setPosition( double x, double y) { centre(0) = x; centre(1) = y; }
	void	setPosition( const Vector &m ) 	{ centre = m; }
	void	setStrength( double s ) 	{ strength = s; }
	void	setContrast( double s ) 	{ strength = s; }
	void	setOrientation( angle a )    { orientation = a; }
	const Vector &getPosition() { return centre; }
	double  getContrast()			{ return strength; }

	corner(void) : prop( corner_t ), centre( 2 ) {}

	virtual void draw( image< rgba< unsigned char > > &output,
                      int i, int j, bool c)
	{
	    int s;
	    if (!c)
		s = 255;
	    else
	    {
		s = (int)(strength);
		if (s>255) s=255;
	    }

	    output.setPixel( i-1,j-1, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i,j-1, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j-1, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i-1,j, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i-1,j+1, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i,j+1, rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j+1, rgba< unsigned char >(0,s,0) );

	}

};

typedef boost::shared_ptr< corner > corner_ptr;

};

#endif
