#ifndef MMVL_CORRELATION_OBSERVER_2D_HH_INCLUDED
#define MMVL_CORRELATION_OBSERVER_2D_HH_INCLUDED

#include <iostream>
#include <string>
#include <vector>

#include "corrcoef.h"
#include "image.h"
#include "hypothesis2D.h"

#define CLAMP(x,y,z) ((x<y) ? (x=y) : ((x>z) ? (x=z) : 0)) //put x into [y;z] range
//put this as a private function

namespace mimas{
  namespace hf{  

    /**
     *@ingroup pf 
     * @brief Performs a cross-correlation product on a part of the current image
     * and a kernel image. The kernel image can be taken in the first
     * image according to the (x,y) location and window size given to
     * the constructor.
     *
     * This class has been designed as simple as possible to provide an example 
     * how to use the hypothesis filter. Optimisation using the fact that the  
     * kernel image is always the same to calculate the coeficient correlation
     * can be easily implemented if needed.
     *
     * @sa corrcoef
     * @author Manuel Boissenin
     * @date june 2003
     *
     * Time-stamp: <2004-08-02 22:50:40 engmb>
     * Copyright (C) 2004 by Manuel Boissenin
     *
     ***********************************************************************/

    template<typename Hypothesis_T, typename Pixel_T>
    class correlation_observer2D
      {
      public:
	class set{
	public:
	  int x;
	  int y;
	  int KER_W;
	  int KER_H;
	  
	  set(int x, int y,int ker_w, int ker_h)
	    :x(x), y(y), KER_W(ker_w), KER_H(ker_h){}
	};
	
	correlation_observer2D(image< Pixel_T > &image, set co_set):
	  image_private(image), KER_W(co_set.KER_W), KER_H(co_set.KER_H)
	{
	  kernel_image.init(co_set.KER_W, co_set.KER_H);
	  set_correlation_observer_members(co_set.x, co_set.y);
	}

	image<int> kernel_image; ///<for display purposes 

	double convolvePoint(const int x, const int y) const;

	void set_correlation_observer_members(const int x,const int y);
	
	void observe(Hypothesis_T &p);

	//!!I put the draw function in the observer at the moment as
	//the observer is more likely to have the information necessary
	//to do it might change latter
	//this function can be used to display the result of analyse from an analyse_result policy
        void draw(const hypothesis2D &h)
	{
	  int DOT_SIZE = 7;
	  int color = 127;
	  
	  std::cerr<<"tracking value is: " <<h.x<< ", " <<h.y << std::endl;

	  for(int i = -DOT_SIZE/2; i< DOT_SIZE/2; i++)
	    for(int j = -DOT_SIZE/2; j< DOT_SIZE/2; j++)
	      {
		image_private.setPixel(int(h.x) + i, int(h.y) + j, Pixel_T(color));  
	      }
	}

	void draw(std::vector<hypothesis2D> &hypotheses)
	{
	  typename std::vector<Hypothesis_T>::iterator it_h;
	  
	  for(it_h = hypotheses.begin(); it_h != hypotheses.end(); it_h++)
	    draw(*it_h);
	}

      private:

	image<Pixel_T> &image_private; ///<reference to the observed image
	int KER_W, KER_H;///<sizes of the correlation window
	
	std::vector<int> kernel;
	
      };
#include "correlation_observer2D.tcc"    
  }
} 
#endif
