#ifndef CORR_TRACKING_TCC
#define CORR_TRACKING_TCC

#include <fstream>
#include <float.h>
#include "image_draw.h"
#include "xml_document.h"

namespace mimas {
template< typename T, class C >
  correlation_tracking_tool< T, C >::correlation_tracking_tool(void)
{
    numPolyPoints = 0;
	templateRows = 0;
	templateCols = 0;
	offsetForFirstCoord.x = 0;
	offsetForFirstCoord.y = 0;
	
    lastTrackingCorr = -FLT_MAX;
    autoUpdateTemplate = false;
    autoUpdateCorrVal = -FLT_MAX;
	trackingRange.x = 0;
	trackingRange.y = 0;
	startPoint.x = 0;
	startPoint.y = 0;
	trackingDispl.x = 0;
	trackingDispl.y = 0;
	
    debug = false;
}

template< typename T, class C >
correlation_tracking_tool< T, C >::~correlation_tracking_tool(void)
{
    //polyCoords.clear();
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::variableDiagnostics(void)
{
    std::cout << "offsetForFirstCoord" << offsetForFirstCoord << std::endl;
}		


template< typename T, class C >
void correlation_tracking_tool< T, C >::setAutoUpdateTemplate(bool val)
{
    autoUpdateTemplate=val;
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::setAutoUpdateCorrVal(double val)
{
    autoUpdateCorrVal=val;
}
	

template< typename T, class C >
void correlation_tracking_tool< T, C >::setTrackingRange(int range)
{
    trackingRange.x = range;
    trackingRange.y = range;
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::setTrackingRange(int xrange, int yrange)
{
    trackingRange.x = xrange;
    trackingRange.y = yrange;
}

template< typename T, class C >
Cartesian<int> correlation_tracking_tool< T, C >::getTrackingPoint(void)
{
	return startPoint;
}

template< typename T, class C >
Cartesian<int> correlation_tracking_tool< T, C >::getTrackingDisplacement(void)
{
	return trackingDispl;
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::pushPolyCoord(int xcoord, int ycoord)
{
	//polyCoords.push_front(xcoord,ycoord);

	Cartesian<int> tempcoord;

	tempcoord.x=xcoord;
	tempcoord.y=ycoord;

	polyCoords.push_back(tempcoord);

	numPolyPoints++;
}


template< typename T, class C >
const std::list< Cartesian<int> > &correlation_tracking_tool< T, C >::getPolyCoords(void) const
{
	return polyCoords;
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::showPolyCoords(void)
{
   for(std::list< Cartesian<int> > ::iterator i=polyCoords.begin();
      i != polyCoords.end(); ++i)
		std::cout << *i << std::endl;
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::clearPolyCoords(void)
{
	polyCoords.clear();
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::sortPolyCoords(void)
{
	polyCoords.sort();
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::findRectTemplate(Cartesian<int> &topLeft, Cartesian<int> &bottomRight)
{
   std::list< Cartesian<int> > ::iterator i;

	int xmax=-1, ymax=-1, xmin=INT_MAX, ymin=INT_MAX;

	for(i=polyCoords.begin(); i != polyCoords.end(); ++i)
	{
		xmax=( (xmax<(*i).x) ? (*i).x:xmax );
		ymax=( (ymax<(*i).y) ? (*i).y:ymax );
		xmin=( (xmin>(*i).x) ? (*i).x:xmin );
		ymin=( (ymin>(*i).y) ? (*i).y:ymin );		
	}
	templateRows=xmax-xmin;
	templateCols=ymax-ymin;

	topLeft.x=xmin; topLeft.y=ymin;
	bottomRight.x=xmax; bottomRight.y=ymax;

	offsetForFirstCoord.x=(*polyCoords.begin()).x-xmin;
	offsetForFirstCoord.y=(*polyCoords.begin()).y-ymin;
}		


template< typename T, class C >
int correlation_tracking_tool< T, C >::pointInPolygon(int x, int y)
{
	std::list< Cartesian<int> > ::iterator i,j;
	int c;

	c=0;

	// ugly logic in the following line but I can't see how to achieve it otherwise!
	for(i=polyCoords.begin(), j=--polyCoords.end() ; i != polyCoords.end(); j=i,++i)
	{
		if (((((*i).y<=y) && (y<(*j).y)) ||
			(((*j).y<=y) && (y<(*i).y))) &&
			(x < ((*j).x - (*i).x) * (y - (*i).y) / ((*j).y - (*i).y) + (*i).x))
			c = !c;
	}

	return c;

}

template< typename T, class C >
void correlation_tracking_tool< T, C >::createTemplate(image<T> &trackingImage)
{
	Cartesian<int> topleft,botright;
	findRectTemplate(topleft,botright);

	// this are actually the starting point in which tracking will start (see function below which makes use of this)
	startPoint.x=topleft.x;
	startPoint.y=topleft.y;

	if (trackingTemplate.initialised())
	{
		trackingTemplate.clear();
		trackingMask.clear();
	}


	trackingTemplate.init(botright.x-topleft.x+1,botright.y-topleft.y+1);
	trackingMask.init(botright.x-topleft.x +1,botright.y-topleft.y+1);

	int x=0, y=0;
	for (int j=topleft.y; j<=botright.y; j++)
	{
		x=0;
		for (int i=topleft.x; i<=botright.x; i++)
		{

			trackingTemplate.setPixel(x,y, trackingImage.getPixel(i,j));
			trackingMask.setPixel(x,y,pointInPolygon(i,j));
			x++;
		}

		y++;
	}			
	
	templateArea=0;
	
	for (int j = 0; j < trackingMask.getHeight(); j++)
		for (int i = 0; i < trackingMask.getWidth(); i++)
			if (trackingMask.getPixel(i,j)) templateArea++;
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::writeMaskedTemplateToFilePGM(char *fn)
{
	image<T> tempImage;
	int i,j;

	tempImage.init(trackingTemplate.getWidth(),trackingTemplate.getHeight());
	for (i=0; i<trackingTemplate.getWidth(); i++)
		for (j=0; j<trackingTemplate.getHeight(); j++)
		{
			if (trackingMask.pixel(i,j))
				tempImage.pixel(i,j) = trackingTemplate.pixel(i,j);
			else
				tempImage.pixel(i,j) = 0;
			}

	tempImage.writeToFilePGM(fn);
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::writeTemplateToFilePGM(char *fn)
{
	trackingTemplate.writeToFile(fn,"PGM");
}


template< typename T, class C >
void correlation_tracking_tool< T, C >::showPolygonOnImage(image<T> &someImage)
{
	std::list< Cartesian<int> > ::iterator i,j;
	for(i=finalPolyCoords.begin(), j=--finalPolyCoords.end() ; i != finalPolyCoords.end(); j=i,++i)
		drawLine(someImage, (*i).x, (*i).y, (*j).x, (*j).y, 127);


}

template< typename T, class C >
void correlation_tracking_tool< T, C >::showPolygonOnImage(image<T> &someImage, int greylevel)
{
	std::list< Cartesian<int> > ::iterator i,j;
	for(i=finalPolyCoords.begin(), j=--finalPolyCoords.end() ; i != finalPolyCoords.end(); j=i,++i)
		drawLine(someImage, (*i).x, (*i).y, (*j).x, (*j).y, greylevel);


}

template< typename T, class C >
Cartesian<int> correlation_tracking_tool< T, C >::trackCorrelation(image<T> &trackingImage, Cartesian<int> &startCoord, Cartesian<int> &range, C &c)
{
/*
	if ((startCoord.x-range.x<0) || (startCoord.x+range.x>=trackingImage.getWidth()) ||
		(startCoord.y-range.y<0) || (startCoord.y+range.y>=trackingImage.getHeight()))
		{
			std::cerr << "ERROR: trackCorrelation::CorrelationTracking - out of range" << std::endl;
			exit(1);
		}
*/
	int i, j;
	int offx, offy;

	Cartesian<int> outCoord;

	double maxcorr=-FLT_MAX;

	for (j=startCoord.y-range.y; j<startCoord.y+range.y; j++)
	{
		for (i=startCoord.x-range.x; i<startCoord.x+range.x; i++)
		{

			if ((j<0 || (j+trackingTemplate.getHeight()>trackingImage.getHeight())) ||
			 	(i<0 || (i+trackingTemplate.getWidth()>trackingImage.getWidth())))
				continue;

         typename C::first_argument_type a;
         typename C::second_argument_type b;

			for (offy=j; offy<j+trackingTemplate.getHeight(); offy++)
				for (offx=i; offx<i+trackingTemplate.getWidth(); offx++)
              if (trackingMask.getPixel(offx-i,offy-j)) {
                a.push_back( trackingImage.getPixel(offx,offy) );
                b.push_back( trackingTemplate.getPixel(offx-i,offy-j) );
              };

         double corr = c( a, b );

			if (maxcorr<corr)
			{
				maxcorr=corr;
				outCoord.x=i;
				outCoord.y=j;
			}

		}
	}

	lastTrackingCorr = maxcorr;

	std::list< Cartesian<int> > ::iterator k;
	finalPolyCoords.clear();
	int offpolyx=(*polyCoords.begin()).x;
	int offpolyy=(*polyCoords.begin()).y;
	
	for(k=polyCoords.begin(); k != polyCoords.end(); ++k) 
	{
		Cartesian<int> tempcoord;

		tempcoord.x=(*k).x-offpolyx+outCoord.x+offsetForFirstCoord.x;
		tempcoord.y=(*k).y-offpolyy+outCoord.y+offsetForFirstCoord.y;

		finalPolyCoords.push_back(tempcoord);			

		if (autoUpdateTemplate)
		    if (autoUpdateCorrVal>lastTrackingCorr)
		    {
			(*k).x=tempcoord.x;
			(*k).y=tempcoord.y;
		    }
	}

	if (autoUpdateTemplate)
	    if (autoUpdateCorrVal>lastTrackingCorr)
         createTemplate(trackingImage);

	trackingDispl.x = outCoord.x - startPoint.x;
	trackingDispl.y = outCoord.y - startPoint.y;
	startPoint = outCoord; // For next search
	
	return outCoord;
}
    
template< typename T, class C > 
Cartesian<int> correlation_tracking_tool< T, C >::trackCorrelation(image<T> &trackingImage, Cartesian<int> &startCoord, C &c) 
{
	return trackCorrelation(trackingImage, startCoord, trackingRange);
} 

template< typename T, class C > 
Cartesian<int> correlation_tracking_tool< T, C >::trackCorrelation(image<T> &trackingImage, C &c) 
{
	return trackCorrelation(trackingImage, startPoint, trackingRange);
} 


template< typename T, class C >
std::list< Cartesian<int> > correlation_tracking_tool< T, C >::trackCorrelationList(image<T> &trackingImage, Cartesian<int> &startCoord, Cartesian<int> &range, C &c)
{
	startCoord.x-=offsetForFirstCoord.x;
	startCoord.y-=offsetForFirstCoord.y;
/*
	if ((startCoord.x-range.x<0) || (startCoord.x+range.x>=trackingImage.getWidth()) ||
		(startCoord.y-range.y<0) || (startCoord.y+range.y>=trackingImage.getHeight()))
		{
			std::cerr << "ERROR: trackCorrelation::CorrelationTracking - out of range" << std::endl;
			exit(1);
		}
*/
	int i, j;
	int offx, offy;

	Cartesian<int> outCoord;

	double maxcorr=FLT_MAX;

	for (j=startCoord.y-range.y; j<startCoord.y+range.y; j++)
	{
		for (i=startCoord.x-range.x; i<startCoord.x+range.x; i++)
		{

			if ((j<0 || (j+trackingTemplate.getHeight()>trackingImage.getHeight())) ||
			 	(i<0 || (i+trackingTemplate.getWidth()>trackingImage.getWidth())))
				continue;
         
         typename C::first_argument_type a;
         typename C::second_argument_type b;
         
			for (offy=j; offy<j+trackingTemplate.getHeight(); offy++)
           for (offx=i; offx<i+trackingTemplate.getWidth(); offx++)
             if (trackingMask.getPixel(offx-i,offy-j)) {
               a.push_back( trackingImage.getPixel(offx,offy) );
               b.push_back( trackingTemplate.getPixel(offx-i,offy-j) );
             };
         
         double corr = c( a, b );
         
			if (maxcorr<corr)
           {
             maxcorr=corr;
             outCoord.x=i;
             outCoord.y=j;
           }
         
		}
	}

	lastTrackingCorr = maxcorr;

	std::list< Cartesian<int> > ::iterator k;
	std::list< Cartesian<int> >  outCoordList;
	int offpolyx=(*polyCoords.begin()).x;
	int offpolyy=(*polyCoords.begin()).y;
	
	for(k=polyCoords.begin(); k != polyCoords.end(); ++k) 
	{
		Cartesian<int> tempcoord;

		tempcoord.x=(*k).x-offpolyx+outCoord.x+offsetForFirstCoord.x;
		tempcoord.y=(*k).y-offpolyy+outCoord.y+offsetForFirstCoord.y;

		outCoordList.push_back(tempcoord);
		
		if (autoUpdateTemplate)
		    if (autoUpdateCorrVal>lastTrackingCorr)
		    {
			(*k).x=tempcoord.x;
			(*k).y=tempcoord.y;
		    }
				
	}

	if (autoUpdateTemplate)
	    if (autoUpdateCorrVal>lastTrackingCorr)
		createTemplate(trackingImage);

	trackingDispl.x = outCoord.x - startPoint.x;
	trackingDispl.y = outCoord.y - startPoint.y;
	startPoint=outCoord; // For next search

	return outCoordList;
}
		

template< typename T, class C >		
double correlation_tracking_tool< T, C >::getTrackingCorr()
{
	return lastTrackingCorr;
}


template< typename T, class C >		
int correlation_tracking_tool< T, C >::getTemplateArea()
{
	return templateArea;
}

#ifdef HAVE_XALAN
template< typename T, class C >		
void correlation_tracking_tool< T, C >::saveParameters( const char *fileName )
{
  std::ofstream ostream( fileName );
  ostream << "<?xml version=\"1.0\"?>" << std::endl
          << "<!DOCTYPE object_parameters SYSTEM \"object_parameters.dtd\">"
	  << std::endl
	  << "<object_parameters>" << std::endl
	  << "<corr_tracking_parameters>" << std::endl
	  << "<range "
	  << "xrange=\"" << (int)trackingRange.x << "\" "
	  << "yrange=\"" << (int)trackingRange.y << "\" "
	  << "/>" << std::endl
	  << "<update "
	  << "auto_update_template=\"" << (int)autoUpdateTemplate << "\" "
	  << "auto_update_template_corr=\"" << (double)autoUpdateCorrVal << "\" "
	  << "/>"  
	  << "</corr_tracking_parameters>" << std::endl
	  << "</object_parameters>" << std::endl;
  MMERROR( ostream, mimasexception, ,
	   "Error saving parameters to file \"" << fileName << "\"" );
}

template< typename T, class C >
void correlation_tracking_tool< T, C >::loadParameters( const char *fileName )
{
  xml_document domDocument( "" );
  domDocument.load( fileName );
  // Removed check on node-name (must be "object_parameters").
  // XSD-file to be provided.
  xml_reference< XalanElement > docElement
    ( domDocument.getDocumentElement() );
  
  xml_reference< XalanElement > n
    ( dynamic_pointer_cast< XalanElement >
      ( domDocument.getDocumentElement().
	selectNode( "corr_tracking_parameters" ) ) );
  
  trackingRange.x = atoi( n.selectNode( "range/@xrange" ).getNodeText().c_str() );
  trackingRange.y = atoi( n.selectNode( "range/@yrange" ).getNodeText().c_str() );

  autoUpdateTemplate =
    (bool)atoi( n.selectNode( "update/@auto_update_template" ).getNodeText().c_str() );
  autoUpdateCorrVal =
    atof( n.selectNode( "update/@auto_update_template_corr" ).getNodeText().c_str() );
}
#endif

}
#endif
