// distributable pairwise
// stuart meikle
// begun Tue Feb 22 10:45:47 2000

// NOTE: an image is just a matrix of 'T' type objects. I'm not going to attempt
// to build in palette information here -> that would be the job of who-ever/whatever
// interprets and renders the image, not a property of the image itself.

// Thu May  4 14:05:13 2000. stu the meikle
// added Iiiiiiiiiiimage Maaaaaagick stuff today. (Its good so you have to say
// like you're introducing it on stage). enables images to be written to file in
// numerous formats.

// Tue Sep 26 13:56:01 - Bala Amavasai
// added readFromFile - read image from file
//			rawData - pointer to the data - useful for directly thunking data in KLT

// Wed Mar 7 19:06:02 - Bala Amavasai
// ImageMagick functions are all F-ed up since versions from 5.2.9 are 16-bit.
// readFromFile seems okay but writeToFile doesn't want to work
// added    readFromFilePGM - read PGM file
//			   writeToFilePGM  - write PGM to file
// the above two functions require netpbm to be installed - note that some buggy installations
// do not include shopts, in which case compile it 'yerself and install it!

// Sat May  4 14:24:14 2002 - Stu Meikle
// some tidying up. made the destructor virtual, which is more useful for images which
// have pointers at each pixel. (edgel images etc).
// addded a switch for objectID, because it's a big structure! (remember histograms
// are images but often not much bigger than 255 bytes).

// SUGGESTION: keep all QT specific code in the image files only. 
// (in case someone needs to remove / switch it in the future)
//
// $Header: /cvs/mimas2/include/image.h,v 1.3 2005/08/16 20:46:51 engmb Exp $
//

#ifndef IMAGE_H
#define IMAGE_H

#include "mimasconfig.h"
#include <boost/multi_array.hpp>
#include <algorithm>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <string>
#include "object.h"
#include "primitives.h"
#include "multi_array_op.h"
#include "mimasexception.h"

#define sgn(x) ((x<0)?-1:((x>0)?1:0))

namespace mimas {
  /**To represent a displacement in an image.
     Used by image iterator
  */
  struct dist2D{
    dist2D(int x = 0, int y = 0):
      width(x), height(y){}
    
    int width;
    int height;
  };

  /** Template class for images.
      Hmmm. Well. image is a generic image class. You specify the pixel type
      at creation time. So if you want an image of ints do
      \code
      image<int> my_image;
      \endcode
      and so on. */
  template<typename T>
    class image: public object
    //perhaps public virtual drawable one day
    {

    protected:
      typedef std::istream istream;
      typedef std::ostream ostream;
      //(co-ordinate system can go in here one day)
      /** Data.
          The image's data is stored in a boost-array. */
      boost::multi_array< T, 2 > data;
      // Where we store out of bounds entries.
      // T	outofbounds;
      T dummy;


    public:

      int getWidth( void ) const
	{
	  return data.shape()[1];
	}
      int getHeight( void ) const
	{
	  return data.shape()[0];
	}

      image( void )
	{
	  dummy = T();
	}

      /// Copy-constructor.
      image( const image<T> &_image ):
      object( _image ), data( _image.data ), dummy( _image.dummy )
	{
	}

      explicit image(const boost::multi_array<T, 2> &array)
	{
	  dummy = T();
	  init(array.shape()[1], array.shape()[0]);
	  data = array;
	}

      template< typename U >
	image( const image< U > &image ) {
	dummy = T();
	init( image.rawData().shape()[1], image.rawData().shape()[0] );
	const U *p = image.rawData().data();
	T *q = rawData().data();
	for ( int i=0; i<(signed)image.rawData().num_elements(); i++ )
	  *q++ = (T)*p++;
      };
      
      virtual void init( int w , int h )
	{
	  // Only resize, if necessary.
	  if ( w != getWidth() || h != getHeight() ) {

	    // Resize to zero first to prevent copying of old data (takes time).
	    data.resize( boost::extents[ 0 ][ 0 ] );
	    data.resize( boost::extents[ h ][ w ] );
	  };
	  
	}

      virtual void fill( const T &initVal ) {
        std::fill( data.data(), data.data() + data.num_elements(),
                   initVal );
      };
        

      virtual void clear( void )
	{
	  data.resize( boost::extents[ 0 ][ 0 ] );
	}

      /** Get const-reference to a pixel.
          This method includes clipping! Specifying out-of-image coordinates
          will result in a const-reference to a dummy pixel.
          @param x x-coordinate
          @param y y-coordinate
          @see pixel method*/
      const T &getPixel( int x, int y ) const
	{
	  if ( x>=0 && y>=0 && x<getWidth() && y<getHeight() )
	    return data[ y ][ x ];
	  else
	    return dummy;
	}

      /** Get reference to a pixel.
          This method includes clipping! Specifying out-of-image coordinates
          will result in a reference to a dummy pixel.
          @param x x-coordinate
          @param y y-coordinate
          @see pixel method*/
      T &getPixel( int x, int y )
	{
	  if ( x>=0 && y>=0 && x<getWidth() && y<getHeight() )
	    return data[ y ][ x ];
	  else {
	    return dummy;
	  };
	}

      /** Set value of a pixel.
          This method includes clipping!
          @param x x-coordinate
          @param y y-coordinate
          @param val New value.
          @see pixel */
      virtual void setPixel( int x, int y, T val )
	{
	  if ( x>=0 && y>=0 && x<getWidth() && y<getHeight() )
	    data[ y ][ x ]  =  val;
	}

      /** Get const-reference to a pixel.
          No clipping!
          @param x x-coordinate
          @param y y-coordinate
          @see getPixel */
      const T& pixel(  int x, int y ) const
	{
	  assert( x>=0 && y>=0 && x<getWidth() && y<getHeight() );
	  return data[ y ][ x ];
	}

      /** Get reference to a pixel.
          No clipping!
          @param x x-coordinate
          @param y y-coordinate
          @see setPixel */
      T& pixel(  int x, int y )
	{
	  assert( x>=0 && y>=0 && x<getWidth() && y<getHeight() );
	  return data[ y ][ x ];
	}

      /// Access to aggregated multi-array.
      boost::multi_array< T, 2 > &rawData( void )
	{
	  return data;
	}

      /// Read-only access to aggregated multi-array.
      const boost::multi_array< T, 2 > &rawData( void ) const
	{
	  return data;
	}
      
      bool initialised( void ) const
	{
	  return data.num_elements() > 0;
	}

      
      /*
	!!!!!! the compiler do a segmentation fault on this code. 
      image<T> subImage(const Pixel ul, const Pixel lr) const
      {
	assert(ul.x < getWidth() && lr.x < getWidth()
	       && ul.y < getHeight() && lr.y < getHeight());
	typedef typename boost::multi_array<T,2>::index_range range;
	typedef boost::multi_array<T,2>::array_view<2>::type mm_sub_array_t;
	mm_sub_array_t subArray =
	  //	    rawData()[boost::indices[range(ul.y, lr.y)][range(ul.x, lr.x)]];
	  rawData()[boost::indices[range(1, 12)][range(32, 45)]];
	return image<T>(boost::multi_array<T,2>(subArray));
      }
      */


      //The upper left corner is included the lower right is excluded
      image<T> sub_image(const mimas::pixel ul, const mimas::pixel lr) const
      {	
	assert(ul.x < (unsigned int)(getWidth()) && lr.x < (unsigned int)(getWidth())
	       && ul.y < (unsigned int)(getHeight()) && lr.y < (unsigned int)(getHeight()));
	assert(ul.x < lr.x && ul.y < lr.y);

	image<T> im;
	im.init(lr.x - ul.x , lr.y - ul.y);
	for(int x = 0; x < im.getWidth();x++) 
	  for(int y = 0; y < im.getWidth();y++)
	    {
	      im.setPixel(x, y, getPixel(x + ul.x ,y + ul.y));		
	    }
	return im;
      }
      
      
      
      /**iterator
	 implementation of iterator for images as described in the paper
	 STL-style generic programming with images
	 by Ullrich Kothe

	 Still not thoroughly tested
	 @author Manuel Boissenin
      */
      class iterator{
      public:
	typedef T value_type;
	typedef value_type * MoveX;

	
	struct MoveY{	  
	  MoveY(unsigned int width = 0)
	    :width(width),offset(0){
	    //	    std::cerr<< "constructor MoveY"<<std::endl;
	  }

	  void operator++(void){
	    offset += width;
	  }

	  //postfix operator. There is no difference as we don't return any value
	  //but avoid any warning message

	  void operator++(int){
	    offset += width;
	  }
	  
	  
	  void operator--(void){
	    offset -= width;
	  }

	  //postfix operator. There is no difference as we don't return any value
	  //but avoid any warning message

	  void operator--(int){
	    offset -= width;
	  }
	  
	  MoveY & operator+=(int n)
	  {
	    offset += width * n;
	    return *this;
	  }

	  MoveY & operator-=(int n)
	  {
	    offset -= width * n;
	    return *this;
	  }
	  
	  int operator-(const MoveY &y2) const
	  {
	    return (offset - y2.offset)/width;
	  }

	  MoveY & operator=(const MoveY &y2) 
	  {
	    offset = y2.offset;
	    width = y2.width;
	    return *this;
	  }
	    
	  bool operator==(const MoveY &y2) const
	  {
	    return (offset == y2.offset) && (width == y2.width);
	  }

	  bool operator<(const MoveY &y2) const
	  {
	    return offset < y2.offset;
	  }

	  unsigned int width;
	  unsigned int offset;
	};
	  
	MoveX x;
	MoveY y;

	  
	iterator(value_type *data = NULL, unsigned int width = 0):
	  x(data), y(width){}
      	
	iterator(const iterator & rhs)
	  {
	    x = rhs.x;
	    y = MoveY(rhs.y.width);  
	    y.offset = rhs.y.offset;
	  }

	iterator & operator=(const iterator & rhs)
	  {
	    //std::cerr<< "operator= iterator"<<std::endl;
	    x = rhs.x;
	    y = rhs.y;
	    return *this;
	  }

	iterator & operator+=(const dist2D & dist)
	  {
	    x += dist.width;
	    y += dist.height;
	    return *this;
	  }

	  
	iterator & operator-=(const dist2D & dist)
	  {
	    x -= dist.width;
	    y -= dist.height;
	    return *this;
	  }
	  
	iterator operator+(const dist2D & dist) const 
	  {
	    iterator k(*this);
	    k += dist;
	    return k;
	  }

	iterator operator-(const dist2D & dist) const
	  {
	    iterator k(*this);
	    k -= dist;
	    return k;
	  }

	//mechanism to check the iterator of both images are of same width.
	dist2D operator-(const iterator & rhs) const
	  {
#ifndef NDEBUG
	    if(y.width != rhs.y.width){
	      std::cerr<<"Image iterator (in image.h): you cannot compare"
		"2 iterators if both images don't have the same width" <<std::endl;
	    }
#endif
	    return dist2D(x - rhs.x, (y.offset - rhs.y.offset)/ rhs.y.width);
	  }

	  
	bool operator==(const iterator & imIt) const
	  {
	    return (x == imIt.x) && (y == imIt);
	  }
      
	value_type & operator*()
	  {
	    return *(x + y.offset);
	  }

	value_type operator*() const 
	  {
	    return *(x + y.offset);
	  }

	value_type &operator[](const dist2D dist)
	  {
	    return *(x + y.offset + dist.width + (signed)( dist.height * y.width ) );
	  }

	value_type operator[](const dist2D dist) const
	  {
	    return *(x + y.offset + dist.width + (signed)( dist.height * y.width ) );
	  }
	  
      };

      class const_iterator{
      public:
	typedef T value_type;
	typedef const value_type * MoveX;

	
	struct MoveY{	  
	  MoveY(unsigned int width = 0)
	    :width(width),offset(0){
	  }

	  void operator++(void){
	    offset += width;
	  }

	  //postfix operator. There is no difference as we don't return any value
	  //but avoid any warning message

	  void operator++(int){
	    offset += width;
	  }
	  
	  
	  void operator--(void){
	    offset -= width;
	  }

	  //postfix operator. There is no difference as we don't return any value
	  //but avoid any warning message

	  void operator--(int){
	    offset -= width;
	  }
	  
	  MoveY & operator+=(int n)
	  {
	    offset += width * n;
	    return *this;
	  }

	  MoveY & operator-=(int n)
	  {
	    offset -= width * n;
	    return *this;
	  }
	  
	  int operator-(const MoveY &y2) const
	  {
	    return (offset - y2.offset)/width;
	  }

	  MoveY & operator=(const MoveY &y2) 
	  {
	    offset = y2.offset;
	    width = y2.width;
	    return *this;
	  }
	    
	  bool operator==(const MoveY &y2) const
	  {
	    return (offset == y2.offset) && (width == y2.width);
	  }

	  bool operator<(const MoveY &y2) const
	  {
	    return offset < y2.offset;
	  }

	  unsigned int width;
	  unsigned int offset;
	};
	  
	MoveX x;
	MoveY y;

	  
	const_iterator(const value_type *data = NULL, unsigned int width = 0):
	  x(data), y(width){}
      	
	const_iterator(const iterator & rhs)
	  {
	    x = rhs.x;
	    y = MoveY(rhs.y.width);  
	    y.offset = rhs.y.offset;
	  }

	const_iterator & operator=(const const_iterator & rhs)
	  {
	    x = rhs.x;
	    y = rhs.y;
	    return *this;
	  }

	const_iterator & operator+=(const dist2D & dist)
	  {
	    x += dist.width;
	    y += dist.height;
	    return *this;
	  }

	  
	const_iterator & operator-=(const dist2D & dist)
	  {
	    x -= dist.width;
	    y -= dist.height;
	    return *this;
	  }
	  
	const_iterator operator+(const dist2D & dist) const 
	  {
	    const_iterator k(*this);
	    k += dist;
	    return k;
	  }

	const_iterator operator-(const dist2D & dist) const
	  {
	    const_iterator k(*this);
	    k -= dist;
	    return k;
	  }

	//mechanism to check the iterator of both images are of same width.
	dist2D operator-(const const_iterator & rhs) const
	  {
#ifndef NDEBUG
	    if(y.width != rhs.y.width){
	      std::cerr<<"const_iterator (in image.h): you cannot compare"
		"2 iterators if both images don't have the same width" <<std::endl;
	    }
#endif
	    return dist2D(x - rhs.x, (y.offset - rhs.y.offset)/ rhs.y.width);
	  }

	  
	bool operator==(const const_iterator & imIt) const
	  {
	    return (x == imIt.x) && (y == imIt);
	  }
      
	const value_type & operator*()
	  {
	    return *(x + y.offset);
	  }

	const value_type operator*() const 
	  {
	    return *(x + y.offset);
	  }

	const value_type &operator[](const dist2D dist) 
	  {
	    return *(x + y.offset + dist.width + dist.height * y.width);
	  }

	const value_type operator[](const dist2D dist) const
	  {
	    return *(x + y.offset + dist.width + dist.height * y.width);
	  }
	  
      };

      inline iterator upperLeft(void)
	{
	  return iterator(data.data(),getWidth());
	}

      inline iterator lowerRight(void)
	{
	  iterator t(data.data(),getWidth());
	  t.x += getWidth();
	  t.y += getHeight();
	  return t;
	}

      //For lazy guys/pals/mates/fellows only!!!
      //the same as uperLeft
      inline iterator ul(void)
	{
	  return iterator(data.data(),getWidth());
	}
      
      //the same as lowerRight
      inline iterator lr(void)
	{
	  iterator t(data.data(),getWidth());
	  t.x += getWidth();
	  t.y += getHeight();
	  return t;
	}


      //to return const values
      inline const_iterator upperLeft(void) const
	{
	  return const_iterator(data.data(),getWidth());
	}

      inline const_iterator lowerRight(void) const
	{
	  const_iterator t(data.data(),getWidth());
	  t.x += getWidth();
	  t.y += getHeight();
	  return t;
	}

      inline const_iterator ul(void) const
	{
	  return const_iterator(rawData().data(),getWidth());
	}
      
      inline const_iterator lr(void) const
	{
	  const_iterator t(data.data(),getWidth());
	  t.x += getWidth();
	  t.y += getHeight();
	  return t;
	}

      image<T>& operator=( const image<T> &_image )
	{
	  dummy = _image.dummy;
	  // Only resize, if size has to change. The content gets overwritten
	  // anyway.
	  if ( _image.getWidth() != getWidth() ||
	       _image.getHeight() != getHeight() )
	    data.resize( boost::extents[ _image.getHeight() ]
			 [ _image.getWidth() ] );
	  data = _image.data;
	  return (*this);
	}
      
    };

}

#endif
