#ifndef MMVL_IMAGE_LOADER_HH_INCLUDED
#define MMVL_IMAGE_LOADER_HH_INCLUDED

#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <image.h>
#include <image_fileinput.h>
#include "mimasexception.h"

namespace mimas{
  namespace hf{
    /**
       \ingroup pf
       A policy (ref Modern C++ Design Andrei Alexandrescu) for hypothesis_filter
       that will take care to load a sequence of images if the image names of the sequence  
       follows the naming convention base_nameXXXX.png, X being a digit.
       To take into account other naming conventions, add a new policy.
    */
    
    template<typename T>
    class image_loader{
    public:
      image<T> &image_IL;
      
      /**This class is used to pass the parametres to the image class loader.
	 According to the policy the number of parametre may change but the constructor remains with one parameter.*/
      class set{
      public:
	std::string basename;
	int nb_images; 
	
	int image_nb;
	std::string extension;
	int size;
	int first_image_number;
	
	set(std::string basename, int nb_images, std::string extension = ".pgm",
	    int size = 4, int first_image_number = 0)
	  :basename(basename), nb_images(nb_images), extension(extension),
	   size(size),first_image_number(first_image_number){} 
      };
      
      //carefull, if it is not used as a policy you need to load the first image by
      //a call to next() which is not very satisfying in the sense that when
      //the class is instanciated it is not directly usable.
      image_loader(image<T> &image, set param)
      	:image_IL(image), basename(param.basename), nb_images(param.nb_images),
	image_nb(param.first_image_number),extension(param.extension),size(param.size)
      {
	next(); //load the first image so the object is ready to use 
      }

      //debug code
      std::string fileName(void)
      {
	std::ostringstream str;
	str << basename << std::setw( size ) << std::setfill( '0' ) <<  image_nb
	    << extension;
#ifndef NDEBUG
	std::cerr<< "File name is: " << str.str() <<std::endl; 
#endif
	return str.str();
      }

      /** load next image of the sequence in image_IL send a mimasexception when max number of image is reached.
       */
      void next(void) throw(mimasexception)
      {
	//put an ending condition
	if(image_nb < nb_images)
	  {
	    std::ifstream cimage_file(fileName().c_str(), std::ios::binary );
	    cimage_file >> image_IL;
	    image_nb++;
	    return;
	  }
	else
	  MMERROR(false , mimasexception, ,
		   "max number of image to read reached");
      }

    private:

      std::string basename;
      int nb_images; ///to store the number of image in the sequence

      int image_nb;
      std::string extension;
      int size;
    };

  }
}

#endif
