#ifndef MMVL_P_CONDENSATION_HH
#define MMVL_P_CONDENSATION_HH

#include "condensation.h"

namespace mimas{
  namespace hf{

    /**@ingroup pf
       @brief a picking policy

       The same as condensation with the possibility to reinitialised part of the hypotheses.
    */
    template<typename Hypothesis_T>    
    class p_condensation: public condensation<Hypothesis_T>
    {
    public:
      
      /**This should be called before the filter stage occured:
	suppose the previous position has been identified  with
	a reasonably good degree of confidence. Then we can relocate 
	around this position the previous hypotheses that had a
	low weight.

	Another way is to look at all hypotheses and partial reinitialise
	around the best one.

	proportion must be between 0 and 1

	!!!research work to be done to automate finding of optimal parametres
      */
      void partial_reset(const Hypothesis_T &init, float proportion, float weight_discrimination, float weight_initialization, bool drift_switch = true)
      {
	unsigned int number_particle_to_relocate =  (unsigned int)(condensation<Hypothesis_T>::hypotheses_CD.size() * proportion); 
	
	unsigned int i = 0;
	unsigned int exit_safety = 0;
	while(i < number_particle_to_relocate)
	  {
	    if(exit_safety >= condensation<Hypothesis_T>::hypotheses_CD.size())
	      {
		break;
		std::cerr<<"Apparently their is not enough hypothesis with a lower weight to be reinitialised"<<std::endl;
	      }
	    int j = int(Dice::uniform_random() * condensation<Hypothesis_T>::hypotheses_CD.size());
	    if( (condensation<Hypothesis_T>::hypotheses_CD.at( j ) ).weight < Dice::uniform_random() * weight_discrimination){ //to take preferably low weighted particles.
	      if(drift_switch)
		condensation<Hypothesis_T>::hypotheses_CD.at( j ) = init.drift();
	      else
		condensation<Hypothesis_T>::hypotheses_CD.at( j ) = init;
	      (condensation<Hypothesis_T>::hypotheses_CD.at( j ) ).weight = weight_initialization; //to have less chance to take this particle again
	      //and to have more chance to pick this at the filter stage.
	      i++;
	    }
	    else
	      exit_safety++;
	  } 	
      }

    protected:
      
      p_condensation(std::vector<Hypothesis_T> &hypotheses)
	  :condensation<Hypothesis_T>(hypotheses){}

    };

  }
}
#endif
