//
// scene image stuff. need a separate class to collate all the
// comparison results etc. 
// 
// stuart meikle 
// Thu May 11 13:05:23 2000
//

// scene objects are also pgh_2dobject objects as there are a flat series of lines
// which are converted into pghs.

// bpa - Tue Apr 17 13:49:36 BST 2001
// Added new addModel function
//	addModel("pgm_filename","model_name")
// NOT WORKING PROPERLY! DO NOT USE

#ifndef PGH_SCENE_H
#define PGH_SCENE_H

#include <boost/shared_array.hpp>
#include <string>
#include <list>
#include <sys/time.h>
#include "mimasconfig.h"
#include "object.h"
#include "pgh_2dobject.h"
#include "object_instance_list.h"
#include "image.h"

//think through the algo a bit:-
//(1) register all the objects we're interested in finding.
//    create histograms for them all and all that
//(2) compare each scene line to each of the lines of each of the objects
//    we need to store the response of each SCENE line to each line in each object
//    (not the response of each object line to each scene line)

//for each scene line
//

//
// NOTE TO DEVELOPERS: the scene class (this one) will take control of all
// pointed to data structures passed in. ie if you pass a pointer to an object
// into addModel then that structure will be deleted when this class is destroyed.
//
namespace mimas {
/**
   representing a visual scene (IE a camera image) as a pgh model so that comparison
   can be made between the scene and the known models. based on original tina tool source
   however the algorithm used here is different. here we simply say, for a given model and
   scene, are the lines in the scene consistent with the model.

   :-). Ok. AN input image from a camera consists of a lot of straight lines.
   From a pairwise perspective this set of lines can be considered as another
   pgh_2dobject, however in this case we typically want to compare other
   2d_objects to the scene and to store and collate the results to find all
   the instances of (eg) arthur the stegosaurus in the scene. Hence the extra
   class. Pgh_scene inherits pgh_2dobject and the base class is used to store
   and update the set of histograms which form the scene itself. On top of
   this the scene class itself contains matrices for comparing all known
   objects etc.
   
   Now! Most of the code here is all my own, so don't blame me if it doesn't
   work :-). Also I've changed the location algorithm quite a bit.

   Use the pgh_scene like this:
   \code
    2dobject	dino1,dino2;
    pgh_scene	s;
    //upload the model info s.addModel(dino1); s.addModel(dino2);
    do {
       //get the new scene lines list<line*>  ll = getLinesfromIntelligentCamera(); 
       s.updateSceneLines( ll, image_width, image_height);
       
       //use it!  object_instance_list oil = s.findModel( dino2 );
    }
    \endcode
    So its pretty straightforwards. Should work ok as in the example.
    
    Obviously this is a little less complex with the original tina, so i'd
    better explain what's going on. Basically you can still use either
    long_lines or points to locate the objects, however at the moment its hard
    coded for points, and i'd recommend that it stays that way. Short lines
    hasn't been implemented yet, and even points i've completely rewrittten.

    So what does happen now? Basically the location stuff is highly geared
    towards the possibility that there might be more than one instance of a
    given model in the scene. Consequently all scene lines are compare to all
    the model lines for the model youre looking for. The results are then
    simply thresholded and then any remaining successful matches are examined
    to find the consisten possible model instances in the scene. A location
    PHT is used as in the original tina, however this time it scales to match
    the size of the input image (!) and also the lines entered into it are
    drawn correctly and have width >1. The points method is more successful
    than the long lines method because each possible pair of lines in the
    scene (or model?) is used and only when a pair of matches is consistent
    with a given single location are points plotted into the location
    histogram. This removes a lot of the problems we have with long lines.

    Basically there are a lots of points entered into a PHT for the possible
    location of model instances in the scene. This is then thresholded and
    the locations of any remaining peaks are taken to be locations of instances
    of the model you're looking for. The user is returned a list of this
    locations wrt the co-ordainte system of the origianl image.

    See the example. Talk to fabio. Email me if you're still confused. If you
    want, you can add/uncomment the hough.display() lines in the process to
    check what the histograms look like. (You might need to scale them first
    though). */
class	pgh_scene : public pgh_2dobject
{
    //NOTE: the input camera scene is a 2d pgh object in its own right.


    /**
     * small structure used to associate match scores with object line and pgh
     */
    class	scene_match_score
    {
        public:
        double	best_match;
        pgh_ptr object_pgh;
        line_ptr object_line;
    };
 
    typedef boost::shared_array< scene_match_score > scene_match_score_array;
    typedef std::list< scene_match_score_array > scene_match_score_array_list;
    typedef pgh_2dobject_ptr_list::iterator			obj_it;
    typedef scene_match_score_array_list::iterator		sms_it;
    typedef enum { long_lines, short_lines, points } 	pgh_loc_type;


    private:
	pgh_2dobject_ptr_list	objects_of_interest;///<list of objects which we are interested in finding in the scene

				//this is essentially like Class definitions
				//	object_instance_list	instances;
				//for each of the objects_of_interest
				//we have a list of all of the found instances
				//(if they have been searched for)

     // std::list<bool>		we_created_obj;///<store bool for each of objects of interest which says if this class created it
				//not all of the object line data will be created by this
				//class -> we shouldn't delete external data!

   scene_match_score_array_list	match_score_list;///<line match scores.each element is an array, scene_match_score[num_scene_lines]
				//each element is scene_match_score[ num_scene_lines ]
				//one entry per object here.

#ifdef HAVE_GETTIMEOFDAY
	struct timeval		timestamp;///< time at which the scene was last updated
#endif
	pgh_loc_type		location_hough_type;

				//info regarding how this set of scene lines relates to the
				//underlying image. non-ideal!

	double			centroid_posn_x;///<position of the centroid of the scene lines on the image plane
	double			centroid_posn_y;///<position of the centroid of the scene lines on the image plane
				//the co-ordinates of the set of scene lines centroid on
				//the original image. a bit weird...

	int			image_height, image_width;///< size of the camera image from which the scene representation is derived
				//size of the attached camera image


	//funcs:
	object_instance_list_ptr locateObject( scene_match_score_array match_array, pgh_2dobject_ptr ob );
	object_instance_list_ptr locateObject_longLines( scene_match_score_array match_array , pgh_2dobject_ptr ob );
	object_instance_list_ptr locateObject_points( scene_match_score_array match_array , pgh_2dobject_ptr ob );
	angle orientObject( scene_match_score_array match_array , pgh_2dobject &ob, const Vector &loc );


    public:
	pgh_scene(void);
	~pgh_scene(void);

	// added by fbc, 15.11.01
	void			clear(void);


	void			addModel( const line_ptr_list &geom,
                         const std::string &name );
	void			addModel( pgh_2dobject_ptr ob );
	// void			addModel( pgh_2dobject *ob, bool wco );

	// following was added by bpa
	// NOT WORKING PROPERLY! DO NOT USE
	void			addModel(char *pgmfilename , const char *modelname );


	object_instance_list_ptr findModel( pgh_2dobject_ptr ob );
	void			updateSceneLines( const line_ptr_list &newscene, int w, int h );

	// added by bpa
	void			updateSceneLines(char *newscenePGM, int w, int h );
	void			updateSceneLines( image<int> &imagein);

	// modified by fbc, 17.12.01
	void			updateSceneLines( image<int> &imagein, char *polyfilename);


	int			getNumScenePGHs(void);
	pgh_ptr_list *getScenePghList(void);
	// shifted here so that user can change value
	bool			prefiltering;
	double			pgh_match_thresh;    //(becoming a bit of a mess i think! -stu)
	double			hough_peak_thresh;
#ifdef HAVE_XALAN
	// added by Jan Wedekind
	// save parameters to xml-file. Returns true on success.
	void saveSceneParameters( const char *fileName );
	// load parameters from xml-file. Returns true on success.
	void loadSceneParameters( const char *fileName );
#endif
};

}

#endif
