//
// stumeikle 
// Mon 28 Oct 2002 20:24:18 CET
//


#ifndef POL_H
#define POL_H

#include "object.h"

namespace mimas {
/**
 *
 * Pairs Of Lines tool. Another shape-based object recognition tool.
 * Original back-of-beer-mat algorithm by stumeikle. pairwise will be
 * better if you have a lot of models, otherwise this is ok. simple
 * matching of pairs of lines from a model(s) and a visual scene. 
 *
 * Algorithm goes as follows: (roughly)
 * extract all possible pairs of lines from a model view. organise into
 * table based on the angles between each pair.
 *
 * Do the same for a visual scene. For each pair of lines in the visual 
 * scene decide if they could be any of the model pairs of lines. If it
 * would be possible then make an entry into a 2D hough transform for the
 * possible centroid position of the model. Continue until all scene pairs
 * of lines have been considered. Extract peaks from the hough and there you
 * go.
 *
 * There's a bit of optimisation included and a novel hough transform is used
 * to estimate the orientation of the found models in the scene. 
 *
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Mon 28 Oct 2002 20:24:18 CET
 * $Header: /cvs/mimas2/include/pol.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */
class   pol
{
    //protected
    /**
    *  just common vars here. static as they should be determined by the
    *  measurement system (ie camera etc) and not by the application
    *  used by both the pol model and the scene
    */
    
protected:
    //static parameters, used for all models
    static int			num_abins;///< how many distinct angles can be measured?
    static double		min_len;///< minimum length of line which should be considered
    static double		len_error;///< lengths can be measured to +- this value
    static double		min_angle_diff;///< parallel lines dont contribute much info. what angles will we consider?
    static double		max_angle_diff;///< parallel lines dont contribute much info. what angles will we consider?
    static double		max_scale; ///<for the scale invariant version
    static double		min_scale; ///<for the scale invariant version
};
}

#endif
