#include "image_gstreamerinput.h"

#if defined(HAVE_SYS_SEM_H) && defined(HAVE_SYS_SHM_H)

#include "rgba.h"

#include <iostream>

using namespace std;

namespace mimas {

template<>
void image_gstreamerinput< rgba< unsigned char > >::read
   ( image< rgba< unsigned char > > &image ) throw (mimasexception)
{
  MMERROR( state, mimasexception, ,
	   "Attempt to read image from image_gstreamerinput, after "
	   "gstreamer-pipeline has terminated." );

  // Decrease read semaphore (blocking).
  struct sembuf buf = { 0, -1, SEM_UNDO };
  if ( semop( semid, &buf, 1 ) == 0 ) {

    // Retrieve size of image.
    const int *size = (int *)shmem;
    image.init( size[0], size[1] );
    
    // Copy data from shared memory and convert from RGB to RGBA.
    // Direct access to image-memory!
    assert( sizeof( rgba< unsigned char > ) == 4 );
    unsigned char *p = (unsigned char *)image.rawData().data();
    const unsigned char *q = (unsigned char *)shmem + 8;
    int numpix = image.rawData().num_elements();
    for ( int i=0; i<numpix; i++ ) {
      *p++ = *q++;
      *p++ = *q++;
      *p++ = *q++;
      *p++ = 0;
    };
    
    // Increase write semaphore.
    struct sembuf buf = { 1, +1, 0 };
    MMERROR( semop( semid, &buf, 1 ) == 0, mimasexception, ,
             "Error switching semaphores with key " << semkey << "." );
  } else {
    state = false;
  };
}

template<>
void image_gstreamerinput< unsigned char >::read
   ( image< unsigned char > &im ) throw (mimasexception)
{
  MMERROR( state, mimasexception, ,
	   "Attempt to read image from image_gstreamerinput, after "
	   "gstreamer-pipeline has terminated." );

  // Decrease read semaphore (blocking).
  struct sembuf buf = { 0, -1, SEM_UNDO };
  if ( semop( semid, &buf, 1 ) == 0 ) {

    // Retrieve size of image.
    const int *size = (int *)shmem;
    image< rgba< unsigned char > > colorImage;
    colorImage.init( size[0], size[1] );
    
    // Copy data from shared memory and convert from RGB to RGBA.
    // Direct access to image-memory!
    assert( sizeof( rgba< unsigned char > ) == 4 );
    unsigned char *p = (unsigned char *)colorImage.rawData().data();
    const unsigned char *q = (unsigned char *)shmem + 8;
    int numpix = colorImage.rawData().num_elements();
    for ( int i=0; i<numpix; i++ ) {
      *p++ = *q++;
      *p++ = *q++;
      *p++ = *q++;
      *p++ = 0;
    };

    im = image< unsigned char >( colorImage );
    
    // Increase write semaphore.
    struct sembuf buf = { 1, +1, 0 };
    MMERROR( semop( semid, &buf, 1 ) == 0, mimasexception, ,
             "Error switching semaphores with key " << semkey << "." );
  } else {
    state = false;
  };
} 

};

#endif

