#include <cassert>
#include "image_mesaoutput.h"

#ifdef HAVE_OPENGL

#include "mesa_onscreen.h"

using namespace std;

namespace mimas {

// Thread-variable.
pthread_t staticThread;

template<>
void image_mesaoutput< rgba< unsigned char > >::image_window::
  paint_event(void) throw (mimasexception)
{


  // After a resize of the window the image-size may be temporarily wrong.
  if ( img.getWidth() == get_width() &&
       img.getHeight() == get_height() ) {

    mesa_onscreen mesa( this );

    mesa.ortho2D();

    glDisable(GL_DITHER);
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( 0, 0 );
    glPixelZoom( 1, -1 );

    unsigned int retraceCount;
    // Wait for vertical retrace
    // http://www.inb.uni-luebeck.de/~boehme/xvideo_sync.html
    glXGetVideoSyncSGI( &retraceCount );
    glXWaitVideoSyncSGI( 2, ( retraceCount + 1 ) % 2, &retraceCount );

    // Direct access to image-memory!
    glDrawPixels( get_width(), get_height(),
		  GL_BGRA, GL_UNSIGNED_BYTE, img.rawData().data() );
    glEnable(GL_DITHER);

  };
};

template<>
void image_mesaoutput< unsigned char >::image_window::
   paint_event(void) throw (mimasexception)
{

  // After a resize of the window the image-size may be temporarily wrong.
  if ( img.getWidth() == get_width() &&
       img.getHeight() == get_height() ) {


    mesa_onscreen mesa( this );

    mesa.ortho2D();

    glDisable(GL_DITHER);
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( 0, 0 );
    glPixelZoom( 1, -1 );

    unsigned int retraceCount;
    // Wait for vertical retrace
    // http://www.inb.uni-luebeck.de/~boehme/xvideo_sync.html
    glXGetVideoSyncSGI( &retraceCount );
    glXWaitVideoSyncSGI( 2, ( retraceCount + 1 ) % 2, &retraceCount );

    // Direct access to image-memory!
    glDrawPixels( get_width(), get_height(),
		  GL_LUMINANCE, GL_UNSIGNED_BYTE, img.rawData().data() );
    glEnable(GL_DITHER);

  };

};

};

#endif

