#include <boost/shared_array.hpp>
#include "image_mplayeroutput.h"

#ifdef HAVE_MKSTEMP

#include "colourspace.h"

using namespace boost;

namespace mimas {


template<>
void image_mplayeroutput< rgba< unsigned char > >::write
  ( const image< rgba< unsigned char > > &image )
{
  MMERROR( image.getWidth() == width && image.getHeight() == height,
           mimasexception, ,
           "image_mplayeroutput< rgba< unsigned char > >::write() - "
           "Image must be of size " << width << 'x' << height << ", but the "
           "size was " << image.getWidth() << 'x' << image.getHeight()
           << '!' );

  const int yv12Size = width * height * 3 / 2;
  shared_array< unsigned char > yv12( new unsigned char[ yv12Size ] );
  rgba_to_yv12( (const char *)image.rawData().data(), width, height,
                (char *)yv12.get() );
  ::write( imgpipe, yv12.get(), yv12Size );
}

template<>
void image_mplayeroutput< unsigned char >::write
  ( const image< unsigned char > &im )
{
  MMERROR( im.getWidth() == width && im.getHeight() == height,
           mimasexception, ,
           "image_mplayeroutput< rgba< unsigned char > >::write() - "
           "Image must be of size " << width << 'x' << height << ", but the "
           "size was " << im.getWidth() << 'x' << im.getHeight()
           << '!' );

  image< rgba< unsigned char > > colourImage( im );

  const int yv12Size = width * height * 3 / 2;
  shared_array< unsigned char > yv12( new unsigned char[ yv12Size ] );
  rgba_to_yv12( (const char *)colourImage.rawData().data(), width, height,
                (char *)yv12.get() );
  ::write( imgpipe, yv12.get(), yv12Size );
}

};

#endif
