//
// line funcs
// stuart meikle Wed Mar  8 16:22:35 2000
//

#include <cmath>
#include "linalg.h"
#include "line.h"

namespace mimas {
	
line::Vector line::getIntersection( const line &l2,
                                          bool *on_this_line,
                                          bool *on_that_line ) const
{
    //find the intersection of the current line with line l2
    //report back if the point of intersection lies on either or both lines
    * on_this_line = * on_that_line = false;

    double d1,d2,a;
    Vector v1(2),v2(2),cp(2),b(2),isct(2);

    v1 = unit( getVector() );

    v2 = unit( l2.getVector() );

    cp(0) = -v2(1);
    cp(1) =  v2(0);

    a = inner_prod( cp, v1 );

    if (fabs(a)< 0.001) a = copysign(0.001,a);

    a = inner_prod( cp, start - l2.getStart() )/a;
    b = v1*a;

    isct = start - b;
    d1 = norm_2(isct - start);
    d2 = norm_2(isct - end);

    if((d1>0.01)&&(d2>0.01)&&(d1<length())&&(d2<length()))
      *on_this_line = true;

    d1 = norm_2(isct - l2.getStart());
    d2 = norm_2(isct - l2.getEnd());

    if((d1>0.01)&&(d2>0.01)&&(d1<l2.length())&&(d2<l2.length()))
      *on_that_line = true;

    return(isct);
}


line::Vector line::getIntersectionAngle( const line &l2, angle *retval) const
{
    //find the intersection of the current line with line l2
    //return the angle between the two in 'retval' and the point of intersection
    //at the usual returned value
    // bool	b1,b2;
    // double 	d1,d2
    double a;
    Vector v1(2),v2(2),cp(2),b(2),isct(2);

    v1 = unit( getVector() );

    v2 = unit( l2.getVector() );

    cp(0) = -v2(1);
    cp(1) =  v2(0);

    a = inner_prod( cp, v1 );

    if (fabs(a)< 0.001) a = copysign(0.001,a);

    a = inner_prod( cp, start - l2.getStart() )/a;
    b = v1*a;

    isct = start - b;

    //now we have the it itsc point, find the angle between the lines
    //result is -M_PI to +M_PI, +ve being anticlockwise on xy plane
    v1 = findMidPoint();    	v2 = l2.findMidPoint();
    v1 = v1-isct;		v2 = v2-isct;

    *retval = mimas::getAngle( v1, v2 );

    return(isct);
}

angle line::getAngle( const line &l ) const
{
    //returns the angle between the two lines from -M_PI to +M_PI,
    //sign indicating -clockwise or +anticlockwise
    Vector v1(2),v2(2);

    v1 = getVector();
    v2 = l.getVector();

    // nope
    double	y = scalar_cross_product( v1, v2 );
    double	x = inner_prod( v1, v2 );

    angle	a = angle(y,x);

    //post process
    if (y<0)
	a = angle( (double)a+M_PI);
    if ((double)a>(M_PI/2))
	a = angle((double)a-M_PI);

    return a;
}

}
