#include "mesa_base.h"

#ifdef HAVE_OPENGL

#include <GL/glut.h>

using namespace std;

namespace mimas {

bool mesa_base::existInstance = false;

mesa_base::mesa_base( x11_display *_display,
		      XVisualInfo *_visualInfo )
  throw (mimasexception):
  display( _display ), visualInfo( _visualInfo ), finishBase( false )
{
  if ( existInstance ) {
    MMERROR( false, mimasexception, ,
             "There must be only one instance of mesa_base at any time." );
  };
  existInstance = true;
}

mesa_base::~mesa_base(void)
{
  finish();
}

void mesa_base::finish(void)
{
  if ( !finishBase ) {
    assert( existInstance );
    visualInfo = NULL;
    existInstance = false;
    display = NULL;
    finishBase = true;
  };
}

void mesa_base::rectangle( int x1, int y1, int x2, int y2 ) const
{
  assert( !finishBase );
  glBegin( GL_LINE_LOOP );
  glVertex2f( x1 + 0.5, y1 + 0.5 );
  glVertex2f( x2 + 0.5, y1 + 0.5 );
  glVertex2f( x2 + 0.5, y2 + 0.5 );
  glVertex2f( x1 + 0.5, y2 + 0.5 );
  glEnd();
}

void mesa_base::text( const string &s ) const
{
  assert( !finishBase );
  for ( int i=0; i<(signed)s.length(); i++ )
    glutStrokeCharacter( GLUT_STROKE_MONO_ROMAN, s[i] );
}

void mesa_base::glWrap( bool condition, const char *message )
  throw (mimasexception)
{
  if ( !condition ) {
    mimasexception e;
    e << message;
    bool first = true;
    GLenum error;
    while ( ( error = glGetError() ) != GL_NO_ERROR ) {
      e << ( first ? ": " : "; " ) << gluErrorString( error );
      first = false;
    };
    throw e;
  };
}

};

#endif

