#include <float.h>
#include "moments_tool.h"

namespace mimas {
	/**
	 * Computation of image moments and Hu invariant moments
	 * @author Bala Amavasai (bala@amavasai.org)
	 * @date Tue Apr 12 16:00:05 BST 2005
	 * $Header: /cvs/mimas2/lib/moments_tool.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $
	 */

	moments_tool::moments_tool(void)
	{
	}


	boost::numeric::ublas::matrix< double >  moments_tool::findMoments(const image< unsigned char > imagein, const unsigned char bglevel, bool binaryImage)
	{
		Matrix moments;
		moments.resize(5,5);
		moments.clear();

		double y1, y2, y3, y4;
		double x1, x2, x3, x4;
		double pixval;

		for ( int j=0; j<imagein.getHeight(); j++)
			for ( int i=0; i<imagein.getWidth(); i++)
			{
				pixval=(double)imagein.getPixel(i,j);
				if (fabs(pixval-bglevel)<=DBL_MIN) continue;
				if (binaryImage) pixval=1.0;

				y1=(double)j;
				y2=y1*y1;
				y3=y2*y1;
				y4=y2*y2;
				x1=(double)i;
				x2=x1*x1;
				x3=x2*x1;
				x4=x2*x2;

				moments(0,0)+=pixval;
				moments(0,1)+=y1*pixval;
				moments(0,2)+=y2*pixval;
				moments(0,3)+=y3*pixval;
				moments(0,4)+=y4*pixval;

				moments(1,0)+=x1*pixval;
				moments(1,1)+=x1*y1*pixval;
				moments(1,2)+=x1*y2*pixval;
				moments(1,3)+=x1*y3*pixval;
				moments(1,4)+=x1*y4*pixval;

				moments(2,0)+=x2*pixval;
				moments(2,1)+=x2*y1*pixval;
				moments(2,2)+=x2*y2*pixval;
				moments(2,3)+=x2*y3*pixval;
				moments(2,4)+=x2*y4*pixval;

				moments(3,0)+=x3*pixval;
				moments(3,1)+=x3*y1*pixval;
				moments(3,2)+=x3*y2*pixval;
				moments(3,3)+=x3*y3*pixval;
				moments(3,4)+=x3*y4*pixval;

				moments(4,0)+=x4*pixval;
				moments(4,1)+=x4*y1*pixval;
				moments(4,2)+=x4*y2*pixval;
				moments(4,3)+=x4*y3*pixval;
				moments(4,4)+=x4*y4*pixval;

			}
		return moments;
	}

	boost::numeric::ublas::vector< double > moments_tool::findHuInvariantMoments(const image< unsigned char > imagein, const unsigned char bglevel, bool binaryImage)
	{
		Vector huMoments;

		huMoments.resize(7,false);
		huMoments.clear();

		Matrix moments;

		moments=findMoments(imagein, bglevel, binaryImage);
		//std::cerr << "Moments" << std::endl << moments << std::endl << std::endl;

		double xmean=moments(1,0)/moments(0,0);
		double ymean=moments(0,1)/moments(0,0);

		Matrix mu(4,4);
		Matrix eta(4,4);
		mu.clear();
		eta.clear();

		mu(0,0) = moments(0,0);
		mu(1,0) = 0;
		mu(0,1) = 0;
		mu(2,0) = moments(2,0) - xmean * moments(1,0);
		mu(0,2) = moments(0,2) - ymean * moments(0,1);
		mu(1,1) = moments(1,1) - ymean * moments(1,0);
		mu(3,0) = moments(3,0) - 3 * xmean * moments(2,0) + 2 * moments(1,0) * xmean * xmean;
		mu(1,2) = moments(1,2) - 2 * ymean * moments(1,1) - xmean * moments(0,2) + 2 * ymean * ymean * moments(1,0);
		mu(2,1) = moments(2,1) - 2 * xmean * moments(1,1) - ymean * moments(2,0) + 2 * xmean * xmean * moments(0,1);
		mu(0,3) = moments(0,3) - 3 * ymean * moments(0,2) + 2 * ymean * ymean * moments(0,1);


		eta(0,0) = mu(0,0);
		eta(1,0) = mu(1,0);
		eta(0,1) = mu(0,1);
		eta(2,0) = mu(2,0) / ( mu(0,0) * mu(0,0));
		eta(0,2) = mu(0,2) / ( mu(0,0) * mu(0,0)); 
		eta(1,1) = mu(1,1) / ( mu(0,0) * mu(0,0));
		eta(3,0) = mu(3,0) /  pow( mu(0,0), 2.5 );
		eta(1,2) = mu(1,2) /  pow( mu(0,0), 2.5 );
		eta(2,1) = mu(2,1) /  pow( mu(0,0), 2.5 );
		eta(0,3) = mu(0,3) /  pow( mu(0,0), 2.5 );


		huMoments(0)=eta(2,0) + eta(0,2);
		huMoments(1)=(eta(2,0)-eta(0,2))*(eta(2,0)-eta(0,2)) + 4*eta(1,1)*eta(1,1);
		huMoments(2)=( eta(3,0) - 3*eta(1,2))* ( eta(3,0) - 3*eta(1,2)) + (3*eta(2,1)-eta(0,3))*(3*eta(2,1)-eta(0,3));
		huMoments(3)=( eta(3,0) + eta(1,2)) * (eta(3,0) + eta(1,2)) + ( eta(2,1) + eta(0,3)) *( eta(2,1) + eta(0,3)) ;
		huMoments(4)=(eta(3,0) - 3*eta(1,2))*(eta(3,0) + eta(1,2)) * ( (eta(3,0)+eta(1,2))*(eta(3,0)+eta(1,2))
				- 3*(eta(2,1)+eta(0,3))*(eta(2,1)+eta(0,3))) + ( 3*eta(2,1) - eta(0,3))*(eta(2,1)+ eta(0,3))
				* ( 3*(eta(3,0) + eta(1,2))*(eta(3,0) + eta(1,2)) - (eta(2,1)+eta(0,3))*(eta(2,1)+eta(0,3)));
		huMoments(5)=(eta(2,0)-eta(0,2))*( (eta(3,0)+eta(1,2))*(eta(3,0)+eta(1,2)) - (eta(2,1) + eta(0,3))*(eta(2,1)+eta(0,3)) )
			+ 4*eta(1,1)*(eta(3,0)+eta(1,2))*(eta(2,1)+eta(0,3));
		huMoments(6)=(3*eta(2,1) - eta(0,3))*(eta(3,0)+eta(1,2))* ( (eta(3,0)+eta(1,2))*(eta(3,0)+eta(1,2))
				- 3 * (eta(2,1)+eta(0,3)) * (eta(2,1)+eta(0,3))) + ( 3*(eta(1,2) - eta(3,0))*(eta(2,1)+eta(0,3)))
				* ( 3*(eta(3,0)+eta(1,2))*(eta(3,0)+eta(1,2)) - (eta(2,1)+eta(0,3))*(eta(2,1)+eta(0,3)) );

		//	std::cerr << "Centralised Moments" << std::endl << eta << std::endl << huMoments(2) << std::endl;

		return huMoments;



	}

}

