#include "image_conv.h"
#include "linalg.h"
#include "opticflow_diff_tool.h"

#ifdef HAVE_LIBLAPACK

namespace mimas {
using namespace std;
using namespace boost::numeric;

opticflow_diff_tool::opticflow_diff_tool(void)
{
	windowdist=13;
	//variance=2.25; // s.d. =1.5
	variance=1.0;
}


opticflow_diff_tool::~opticflow_diff_tool(void)
{
}


void opticflow_diff_tool::smoothGaussian(image<double>& imagein, double variance)
{
	const int fsize=11;
	
	image<double> f; f.init(fsize,fsize);
	double divcoef = 0.0;

	for (int j=-fsize/2; j<=fsize/2; j++)
	{
		for (int i=-fsize/2; i<=fsize/2; i++)
		{
        double value = exp( -(j*j+i*i)/(2*variance) );
        f.pixel( i+fsize/2, j+fsize/2 ) = value;
        divcoef += value;
		//	cout << f[i+fsize/2][j+fsize/2] << "\t" ;
		}
		//cout << endl;
	}


	imagein = correlate( imagein, f / divcoef );
}



void opticflow_diff_tool::findFlow(image<int> &image_t, image<int> &image_dt)
{
	MMERROR( (image_t.getWidth()==image_dt.getWidth()) &&
            (image_t.getHeight()!=image_dt.getHeight()), mimasexception, ,
            "opticflow_diff_tool::findFlow - image sizes are inconsistent!" );

	image<double>
     it( image_t ),
     idt( image_dt );

	//findFlowLucasKanade(it,idt);

}


void opticflow_diff_tool::findFlow(vector < image<double> > &imagein)
{
   MMERROR( imagein.size()==5, mimasexception, ,
            "opticflow_diff_tool::findFlowLucasKanade - need 5 images in "
            "vector list!" );

	//for (int d=0; d<imagein.size(); d++)
	//	imagein[d].display();

	image<double> curframe;

	curframe=imagein[2];
	//smoothGaussian(curframe,variance);

	//curframe.display();

	image<double> gradx, grady, gradt;
	double gx,gy, gt;
	int gsize=5;
	double D5[5]={-1.0/12.0, 8.0/12.0, 0.0/12.0, -8.0/12.0, 1.0/12.0};

	gradx.init(curframe.getWidth(), curframe.getHeight());
	grady.init(curframe.getWidth(), curframe.getHeight());
	gradt.init(curframe.getWidth(), curframe.getHeight());
	
	for (int j=gsize/2; j<gradx.getHeight()-gsize/2; j++)
		for (int i=gsize/2; i<gradx.getWidth()-gsize/2; i++)
		{
			gx=0.0;
			gy=0.0;
			for (int k=-gsize/2; k<=gsize/2; k++)
			{
				gx+=curframe.pixel(i+k,j)*D5[k+gsize/2];
				gy+=curframe.pixel(i,j+k)*D5[k+gsize/2];
			}
			
			gradx.pixel(i,j) = gx;
			grady.pixel(i,j) = gy;

			// seperated just in case I use a bigger mask later!
			gt=0.0;
			for (int k=0; k<5; k++)
				gt+=imagein[k].getPixel(i,j)*D5[k];
			//gt=imagein[2].getPixel(i,j)-imagein[1].getPixel(i,j);
			
			gradt.pixel(i,j)=gt;

			
		}
	
	smoothGaussian(gradx,variance);
	smoothGaussian(grady,variance);
	smoothGaussian(gradt,variance);

	// Display gradients in x, y and t directions
	//image<double> tempdisp;
	//tempdisp=gradx;
	//normaliseIt(tempdisp);
	//tempdisp.display();
	//tempdisp.clear();
	//tempdisp=grady;
	//normaliseIt(tempdisp);
	//tempdisp.display();
	//tempdisp.clear();
	//tempdisp=gradt;
	//normaliseIt(tempdisp);
	//tempdisp.display();
		
	xflow.init(gradt.getWidth(),gradt.getHeight());
	yflow.init(gradt.getWidth(),gradt.getHeight());
	
   int
     winSize = 2 * ( windowdist / 2 ) + 1,
     nRows = winSize * winSize;

   ublas::matrix< double, ublas::column_major > A( nRows, 2 );
   ublas::vector< double > b( nRows );

	for (int y=windowdist/2; y<gradt.getHeight()-windowdist/2; y+=windowdist)
		for (int x=windowdist/2; x<gradt.getWidth()-windowdist/2; x+=windowdist)
		{		

			int rowcount=0;
			for (int j=-windowdist/2; j<=windowdist/2; j++)
				for (int i=-windowdist/2; i<=windowdist/2; i++)
				{
					A( rowcount, 0 ) = gradx.pixel(i+x,j+y);
					A( rowcount, 1 ) = grady.pixel(i+x,j+y);
					b( rowcount ) = -gradt.pixel(i+x,j+y);

					rowcount++;

				}

         assert( rowcount == nRows );         

         ublas::vector< double > flow( gels( A, b ) );
         xflow.pixel(x,y) = flow(0);
         yflow.pixel(x,y) = flow(1);

		}


}







// get the x direction vector flow
void opticflow_diff_tool::getXflow(image<double>& xf)
{
	xf=xflow;
}
	
// get the y direction vector flow
void opticflow_diff_tool::getYflow(image<double>& yf)
{
	yf=yflow;
}

}

#endif
