//
// stuart meikle
// Tue Jul  4 10:21:43 2000
// (reorganising .h file into .cc and .h file today)
//

//
// changed to remove dependencies on geompoly Mon 04 Nov 2002 11:44:21 CET
// by stumeikle. (I can't say its 100% checked for mem leaks!)
//
//

#include <boost/shared_ptr.hpp>
#include <fstream>
#include <string>
#include <list>
#include "mimasconfig.h"
#include "pgh.h"
#include "pgh_2dobject.h"
#include "line.h"
#include "image.h"
#include "image_fileinput.h"
#include "canny_tool.h"
#include "slap_tool.h"
#include "edge_string.h"
#ifdef HAVE_XALAN
#include "xml_document.h"
#endif

namespace mimas {
using namespace std;
using namespace boost;

  /* template<class a>
     void	list_destroy(list<a>* b); */

void pgh_2dobject::init(void)
{
    name = "undefined";
    // poly_data = NULL;
    // poly_data_GUI = NULL;
    num_pghs = 0;

	// Canny parameters;
	canny_sigma = 1.0;
	canny_precision = 0.01;
	canny_lowthre = 3.0;
	canny_upthre = 5.0;
	canny_lenthre = 5;

	// ROI parameters;
	roi_lx = 0;
	roi_ly = 0;
	roi_ux = 0;
	roi_uy = 0;

    // these should be derived from the pgh class...I think, fbc 11.10.2001
   	pgh_pairs_type = pgh_type_directed;
   	pgh_dbin_max = 50.0;
   	pgh_dbin_size = 5.0;
   	pgh_dist_ramp = 5.0;
   	pgh_num_abin = 32;
   	pgh_angle_sigma = M_PI/32.0;

	// minimum line length
	min_line_length = 5.0;

	// object tracking region (displacement from its ref. point), fbc, added by 5.12.2001
	track_roi_lx = 0;
	track_roi_ly = 0;
	track_roi_ux = 0;
	track_roi_uy = 0;
}

/**
 * destructor. free up allocated structures
 */
pgh_2dobject::~pgh_2dobject(void)
{
	deleteLineInfo();
}


// added by fbc, 14.01.02 - to be thoroughly tested for memory leaks....
/**
 * clear deletes any allocated structures, so that the object can be used again,
 * (presumably) without having to reset all the parameters
 */
void	pgh_2dobject::clear(void)
{
	name = "undefined";
	deleteLineInfo();
}


/*
 * deletes the list of stored PGHs
 */
void	pgh_2dobject::deletePghList(void)
{
    //delete all the pghs
//     pgh_ptr_list::iterator	i;
//     for( i=pghlist.begin(); i!=pghlist.end(); ++i)
//     {
// 		pgh *l = (*i);
// 		delete l;
//     }

    pghlist.clear();
}

/**
 * deletes the stored poly (that is, line) information.
 */
void	pgh_2dobject::deleteLineInfo(void)
{
    if (num_pghs > 0)
	{
		deletePghList();
	}

    num_pghs = 0;

// if (poly_data)
//    {
//     	line_ptr_list::iterator listit;

// 		for(listit = poly_data->begin(); listit!= poly_data->end(); listit++)
//         {
// 			if (*listit) delete *listit;
//         }

//         poly_data->erase( poly_data->begin(), poly_data->end()   );
//    	delete poly_data;
//   }

//    poly_data=NULL;
    poly_data.reset();

	// added by bpa
//     if (poly_data_GUI)
//     {
//     	line_ptr_list::iterator listit;

// 		for(listit = poly_data_GUI->begin(); listit!= poly_data_GUI->end(); listit++)
// 		{
// 			if (*listit) delete *listit;
// 		}

//       poly_data_GUI->erase( poly_data_GUI->begin(), poly_data_GUI->end()   );

//     	delete poly_data_GUI;
//     }

//     poly_data_GUI=NULL;
    poly_data_GUI.reset();
}

/**
 * returns the name of the object
 */
string	pgh_2dobject::getName(void)
{
    return name;
}

/**
 * returns the number of pghs which comprise the model/object
 */
int	pgh_2dobject::getNumPghs(void)
{
    return num_pghs;
}

/**
 * returns the type of pgh used to represent the lines of the object
 */
pgh_type	pgh_2dobject::getPghType(void)
{
    return pgh_pairs_type;
}

/**
 * used to set the name of the object
 */
void	pgh_2dobject::setName(const string &n)
{
    name = n;
}

/**
 * returns a pointer to the list of pghs which comprise the object
 */
pgh_ptr_list* pgh_2dobject::getPghList(void)
{
    return &pghlist;
}

/**
 * returns a pointer to the list of straight line segments which form the
 * object on the image plane.
 */
line_ptr_list_ptr pgh_2dobject::getListofLines(void)
{
    return poly_data;
}

// added by bpa
/**
 * returns a pointer to the list of straight line segments which form the
 * object on the image plane. What is the difference to the getListofLines function anyone??
 */
line_ptr_list_ptr pgh_2dobject::getListofLinesGUI(void)
{
    return poly_data_GUI;

}


// added by fbc, 6.12.01
/**
 * create the pgh object from an image file. this means load the image, extract edge strings,
 * approximate edge strings with straight lines and then form a PGH for each of the lines in the
 * image.
 *
 */
void	pgh_2dobject::createFromPgmFile(char *pgmfilename)
{
    double	dx, dy;

    createFromPgmFile(pgmfilename, &dx, &dy, "");
    //don't care about dx,dy from this entry point
}


// added by fbc, 17.12.01
/**
 * as for createFromPgmFile , with a specified centroid also passed .
 */
void	pgh_2dobject::createFromPgmFile(char *pgmfilename, double *cx, double *cy)
{
	createFromPgmFile(pgmfilename, cx, cy, "");
}


// added by bpa
/**
 * as for createFromPgmGile with specified centroid and with a filename specified which
 * is used to store the found straight lines as a 'poly file'
 */
void	pgh_2dobject::createFromPgmFile(char *pgmfilename, double *cx, double *cy, char *polyfilename)
{
    image<int>	im;
    ifstream file( pgmfilename, ios::binary );
    file >> im;
    createFromImage(im,cx,cy);

/*
	line_ptr_list *geom = new line_ptr_list;
	line	*l;
	Poly_Lines poly_lines;

	poly_lines.No_Lines = 0;
	poly_lines.Lines = NULL;

	setName(pgmfilename);


	if (pgm2lines(pgmfilename, polyfilename,
			canny_sigma, canny_precision, canny_lowthre, canny_upthre, canny_lenthre,
			pgh_angle_sigma,
			0, 0, 0, 0,
			&poly_lines) != 0)
	{
		mm->Error() << "Lines from model could not created by pgm2lines()" << endl;
		return;
	}


	for (int Index = 0; Index < poly_lines.No_Lines; Index++)
	{
		l = new line(poly_lines.Lines[Index].pt1.X,
						poly_lines.Lines[Index].pt1.Y,
						poly_lines.Lines[Index].pt2.X,
						poly_lines.Lines[Index].pt2.Y);

		geom->push_back(l);
	}


	createFromLines(geom, cx, cy);
	list_destroy<line*>(geom);

	createFromLines(geom, cx, cy);
	LIST_DESTROY( line*, geom );

	if (poly_lines.Lines)
	{
		free(poly_lines.Lines);
		poly_lines.Lines = NULL;
	}
*/
}


// added by fbc, 6.12.01
/**
 * create the object from a list of straight line segments read from the filename given.
 */
void	pgh_2dobject::createFromPolyFile(char *polyfilename)
{
    double	dx, dy;

    createFromPolyFile(polyfilename, &dx, &dy);
    //don't care about dx,dy from this entry point
}


/**
 * as for createFromPolyFile above, however the (found) centroid of the lines is returned
 * in cx and cy.
 *
 */
void	pgh_2dobject::createFromPolyFile(char *polyfilename, double *cx, double *cy)
{
//
// removed by stu. Poly_Lines is in geompoly so i've removed this.
// i guess it should be easy to rewrite, and i see that we've got a
// poly lines tool in mimas anyway ....???
//

/*
	line_ptr_list *geom = new line_ptr_list;
	line	*l;
	Poly_Lines poly_lines;

	poly_lines.No_Lines = 0;
	poly_lines.Lines = NULL;

	setName(polyfilename);

	if (poly2lines(polyfilename, &poly_lines) != 0)
	{
		mm->Error() << "Lines from model could not created by poly2lines()" << endl;
		return;
	}


	for (int Index = 0; Index < poly_lines.No_Lines; Index++)
	{
		l = new line(poly_lines.Lines[Index].pt1.X,
						poly_lines.Lines[Index].pt1.Y,
						poly_lines.Lines[Index].pt2.X,
						poly_lines.Lines[Index].pt2.Y);

		geom->push_back(l);
	}

	createFromLines(geom, cx, cy);
	LIST_DESTROY( line*, geom );

	if (poly_lines.Lines)
	{
		free(poly_lines.Lines);
		poly_lines.Lines = NULL;
	}
*/
}


// added by bpa
//removed by stu.
//we need a non-geom-poly replacement i think
/*
void	pgh_2dobject::createFromStructCoord(struct co_ord *outline, int numcoords)
{

	line_ptr_list *geom = new line_ptr_list;
	line	*l;

    for (int i = 0; i < numcoords - 1; i += 2)
	{
		l = new line(outline[i].X, outline[i].Y, outline[i+1].X, outline[i+1].Y);
		geom->push_back(l);
    }

    createFromLines(geom);

	LIST_DESTROY( line*, geom );

}
*/

// added by fbc
/**
 *
 * create the pgh object from an image. this means extract edge strings,
 * approximate edge strings with straight lines and then form a PGH for each of the lines in the
 * image.
 *
 */
void	pgh_2dobject::createFromImage( image<int> &imagein )
{
    double	dx, dy;

    createFromImage( imagein, &dx, &dy, "" );
    //don't care about dx,dy from this entry point
}


// added by fbc, 17.12.01
/**
 * as for createFromImage, however the found centroid of the lines is also returned, via
 * the variables cx and cy.
 */
void	pgh_2dobject::createFromImage( image<int> &imagein, double *cx, double *cy )
{
    createFromImage( imagein, cx, cy, "" );
}


/**
 * as for createFromImage() with the centroid of the found lines returned via cx,cy and with
 * the discovered straight line segments stored in the specified (poly) filename.
 */
void	pgh_2dobject::createFromImage( image<int> &imagein, double *cx, double *cy, char *polyfilename )
{

   line_ptr_list geom;
	// line	*l;
/*
	Poly_Lines poly_lines;

	poly_lines.No_Lines = 0;
	poly_lines.Lines = NULL;

	if (image2lines(imagein.rawData(), imagein.getWidth(), imagein.getHeight(), polyfilename,
					canny_sigma, canny_precision, canny_lowthre, canny_upthre, canny_lenthre,
					pgh_angle_sigma,
					0, 0, 0, 0,
					&poly_lines) != 0)
	{
		mm->Error() << "Lines from model could not created by image2lines()" << endl;
		return;
	}


	for (int Index = 0; Index < poly_lines.No_Lines; Index++)
	{
		l = new line(poly_lines.Lines[Index].pt1.X,
						poly_lines.Lines[Index].pt1.Y,
						poly_lines.Lines[Index].pt2.X,
						poly_lines.Lines[Index].pt2.Y);

		geom->push_back(l);
	}
*/
	canny_tool<int>		canny;
	slap_tool			slapper;
	edge_string_ptr_list		edge_strings;

   edge_strings=canny.findEdgeStrings( imagein );
	geom = slapper.findStraightLines( edge_strings );

	createFromLines( geom, cx, cy);

	//i probably need to delete the edge strings too

/*
	createFromLines(geom, cx, cy);
	LIST_DESTROY( line*, geom );

	if (poly_lines.Lines)
	{
		free(poly_lines.Lines);
		poly_lines.Lines = NULL;
	}
*/
}

/**
 * create the object from a list of straight line segments.
 * as for the other creation functions, a single pgh will be created for each straight line
 * segment in the list
 */
void	pgh_2dobject::createFromLines( const line_ptr_list &geom )
  throw (mimasexception)
{
    double dx, dy;

    createFromLines( geom, &dx, &dy );
    //don't care about dx,dy from this entry point
}


/**
 * as for createFromLines. the centroid of the lines is returned in cx and cy
 */
void	pgh_2dobject::createFromLines( const line_ptr_list &geom, double *cx, double *cy) throw (mimasexception)
{
    //WARNING: the GEOM structure is COPIED, so it should be
    //deleted afterwards

    //create histograms from each of the lines
    //store these histograms in our list

    #ifdef MODS
    cout << "stu - put a warning in here if the geom is >>> larger than the dbin_max" << endl;
    #endif

    //first preprocess the lines:
    //find the midpoint of each line and then calculate the centroid
    //for the object based upon the weighted mean of these mid points
    double	sum_x, sum_y;
    double	sum;
    double	lx, ly, ux, uy;
    double	weight;

    //(this is more-or-less straight tina code)
    sum_x = sum_y = 0.0;
    sum = 0.0;
    lx = ly = +10e6;
    ux = uy = -10e6;

    //find a centroid for the list of lines and centre the line data
    //about this point when we add it. this immediately destroys any
    //chance we have to relate the object back to the underlying image
    //:-(
    for(line_ptr_list::const_iterator listit=geom.begin();
        listit != geom.end(); listit++ ) {
		line_ptr l = *listit;
		Vector midpoint(2);

		weight = l->length(); // Added apa 11/1/96
		sum += weight;
		midpoint = l->findMidPoint();

		//this doesnt need to be accurate
		if (midpoint(0) < lx) lx = midpoint(0);
		if (midpoint(0) > ux) ux = midpoint(0);
		if (midpoint(1) < ly) ly = midpoint(1);
		if (midpoint(1) > uy) uy = midpoint(1);

		sum_x += midpoint(0) * weight;
      sum_y += midpoint(1) * weight;
    }

    bounding_box_x = (int)(ux - lx);
    bounding_box_y = (int)(uy - ly);

    MMERROR( sum > 0.0, mimasexception, , "There's something wrong with your "
             "line data! Sum of line lengths= zero!!" );

		sum_x /= sum;
		sum_y /= sum;

		//store the results externally
		*cx = sum_x;
		*cy = sum_y;

		Vector modv( 2 ); modv(0) = -sum_x; modv(1) = -sum_y;

		poly_data = line_ptr_list_ptr( new line_ptr_list );
		poly_data_GUI = line_ptr_list_ptr( new line_ptr_list );

		for( line_ptr_list::const_iterator listit = geom.begin();
           listit != geom.end(); listit++ ) {
	    	const Vector *start, *end;
	    	line_ptr l = *listit;

	    	start = &l->getStart();
	    	end   = &l->getEnd();

			if (l->length() <= min_line_length) continue;       // get rid of short lines, fbc 22.10.01

	    	line_ptr
           nl( new line( *start + modv, *end+modv ) ),
           nl_GUI( new line( *start, *end ) );

		    poly_data->push_back( nl );
		    poly_data_GUI->push_back( nl_GUI );
		}

    int count = 0;
    for( line_ptr_list::iterator listit = poly_data->begin();
         listit != poly_data->end(); listit++, count++ )
    {
		pgh_ptr p( new pgh );
		line_ptr l = *listit;

		p->initialise(pgh_pairs_type, pgh_dbin_max, pgh_dbin_size, pgh_dist_ramp, pgh_num_abin, pgh_angle_sigma);

		p->describeLine(*l, *poly_data);
		pghlist.push_back( p );
    }

    num_pghs = count;

//	mm->Message() << "Number of PGHs before trimming : " << geom->size() << "  after trimming : " << poly_data->size() << endl;
}

/**
 * display each of the pghs on screen. used for debugging purposes.
 */
// void	pgh_2dobject::showAll(void)
// {
//     //for debugging really
//     pgh_ptr_list::iterator i;

//     for(i=pghlist.begin(); i!=pghlist.end(); ++i)
//     {
// 		pgh_ptr p = *i;
// 		p->display();
//     }
// }

#ifdef HAVE_XALAN
/**
 * save the object parameters to the given filename. file format is ?? XML
 */
void pgh_2dobject::saveObjectParameters( const char *fileName )
{
   ofstream ostream( fileName );

	ostream << "<?xml version=\"1.0\"?>" << endl
			<< "<!DOCTYPE object_parameters SYSTEM \"object_parameters.dtd\">"
			<< endl
			<< "<object_parameters>" << endl
			<< "<pgh_recognition_parameters>" << endl;

	storeObjectParams( ostream );

	ostream << "</pgh_recognition_parameters>" << endl
		<< "</object_parameters>" << endl;

   MMERROR( ostream, mimasexception, ,
            "Error writing to file \"" << fileName << "\"" );
}

/**
 * load in object parameters from the named file.
 */
void pgh_2dobject::loadObjectParameters( const char *fileName )
{
   xml_document domDocument( "" );
   // Omitting check on node-name (must be "object_parameters").
   // XSD-file to be written.
   domDocument.load( fileName );
   xml_reference< XalanElement > docElement
     ( domDocument.getDocumentElement() );
   extractObjectParams( docElement );
}

/**
 * save object parameters to the given output stream. format is ?? XML
 */
void pgh_2dobject::storeObjectParams( ostream &stream )
{
	stream << "<roi "
			<< "roi_lx=\"" << roi_lx << "\" "
			<< "roi_ly=\"" << roi_ly << "\" "
			<< "roi_ux=\"" << roi_ux << "\" "
			<< "roi_uy=\"" << roi_uy << "\" "
			<< "/>" << endl
			<< "<canny "
			<< "sigma=\"" << canny_sigma << "\" "
			<< "precision=\"" << canny_precision << "\" "
			<< "lowthre=\"" << canny_lowthre << "\" "
			<< "upthre=\"" << canny_upthre << "\" "
			<< "lenthre=\"" << canny_lenthre << "\" "
			<< "/>" << endl
			<< "<pgh "
			<< "pairs_type=\"" << (int)pgh_pairs_type << "\" "
			<< "dbin_max=\"" << pgh_dbin_max << "\" "
			<< "dbin_size=\"" << pgh_dbin_size << "\" "
			<< "dist_ramp=\"" << pgh_dist_ramp << "\" "
			<< "num_abin=\"" << pgh_num_abin << "\" "
			<< "angle_sigma=\"" << pgh_angle_sigma << "\" "
			<< "/>" << endl
			<< "<refpoint "
			<< "x_displ=\"" << ref_point_displ(0) << "\" "
			<< "y_displ=\"" << ref_point_displ(1) << "\" "
			<< "/>" << endl
			<< "<trackingroi "
			<< "track_roi_lx=\"" << track_roi_lx << "\" "
			<< "track_roi_ly=\"" << track_roi_ly << "\" "
			<< "track_roi_ux=\"" << track_roi_ux << "\" "
			<< "track_roi_uy=\"" << track_roi_uy << "\" "
			<< "/>" << endl
			<< "<other "
			<< "modelname=\"" << getName().c_str() << "\" "
			<< "min_line_length=\"" << min_line_length << "\" "
			<< "/>" << endl;
}

/**
 * extract the object parameters from the referenced document element.
 *
 */
void pgh_2dobject::extractObjectParams
   ( xml_reference< XalanElement > &docElement )
{
  xml_reference< XalanElement > n
    ( dynamic_pointer_cast< XalanElement >
      ( docElement.selectNode( "pgh_recognition_parameters" ) ) );

  roi_lx = atoi( n.selectNode( "roi/@roi_lx" ).getNodeText().c_str() );
  roi_ly = atoi( n.selectNode( "roi/@roi_ly" ).getNodeText().c_str() );
  roi_ux = atoi( n.selectNode( "roi/@roi_ux" ).getNodeText().c_str() );
  roi_uy = atoi( n.selectNode( "roi/@roi_uy" ).getNodeText().c_str() );

  canny_upthre = atof( n.selectNode( "canny/@upthre" ).getNodeText().c_str() );
  canny_lowthre = atof( n.selectNode( "canny/@lowthre" ).
                        getNodeText().c_str() );
  canny_lenthre = atoi( n.selectNode( "canny/@lenthre" ).
                        getNodeText().c_str() );
  canny_precision = atof( n.selectNode( "canny/@precision" ).
                          getNodeText().c_str() );
  canny_sigma = atof( n.selectNode( "canny/@sigma" ).getNodeText().c_str() );

  pgh_dist_ramp = atof( n.selectNode( "pgh/@dist_ramp" ).
                        getNodeText().c_str() );
  pgh_dbin_max = atof( n.selectNode( "pgh/@dbin_max" ).getNodeText().c_str() );
  pgh_angle_sigma = atof( n.selectNode( "pgh/@angle_sigma" ).
                          getNodeText().c_str() );
  pgh_num_abin = atoi( n.selectNode( "pgh/@num_abin" ).getNodeText().c_str() );
  pgh_dbin_size = atof( n.selectNode( "pgh/@dbin_size" ).
                        getNodeText().c_str() );
  pgh_pairs_type = (pgh_type)atoi( n.selectNode( "pgh/@pairs_type" ).
                                      getNodeText().c_str() );

  ref_point_displ(0) = atof( n.selectNode( "refpoint/@x_displ" ).
                             getNodeText().c_str() );
  ref_point_displ(1) = atof( n.selectNode( "refpoint/@y_displ" ).
                             getNodeText().c_str() );

  track_roi_lx = atoi( n.selectNode( "trackingroi/@track_roi_lx" ).
                       getNodeText().c_str() );
  track_roi_ly = atoi( n.selectNode( "trackingroi/@track_roi_ly" ).
                       getNodeText().c_str() );
  track_roi_ux = atoi( n.selectNode( "trackingroi/@track_roi_ux" ).
                       getNodeText().c_str() );
  track_roi_uy = atoi( n.selectNode( "trackingroi/@track_roi_uy" ).
                       getNodeText().c_str() );

  setName( n.selectNode( "other/@modelname" ).getNodeText() );
  min_line_length = atof( n.selectNode( "other/@min_line_length" ).
                          getNodeText().c_str() );
}
#endif

}
