//
// property image (pointer image) functions
//
// stumeikle Sun May 12 20:25:53 2002
//
// $Header: /cvs/mimas2/lib/property_image.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $
//


#include "mimasconfig.h"
#include "edge_string.h"
#include "property_image.h"

using namespace std;

namespace mimas {

#ifdef HAVE_OPENGL
image_mesaoutput< rgba< unsigned char > > &operator<<
  ( image_mesaoutput< rgba< unsigned char > > &output,
    const property_image &im )
{
    //display the properties with a image<int>
    image< rgba< unsigned char > > temp;
    temp.init( im.getWidth(), im.getHeight() );
    
    int i,j;
    for(j=0;j<temp.getHeight();j++)
    {
	for(i=0;i<temp.getWidth();i++)
	{
       prop_ptr p( im.pixel(i,j) );
		
	    if( p )
	    {
         p->draw( temp, i, j, im.getDrawContrast() );
	    }
	}
    }
	    
    //now display the image. repeating code here. naughty!
    output << temp;
    return output;
}
#endif

void	property_image::addStrings( edge_string_ptr_list& edge_strings )
{
    //read the strings in and add to the image

    MMERROR( initialised(), mimasexception, ,
             "Can't add strings to an uninitialised image!" );
	
    edge_string_ptr_list::iterator	sit;

    for(sit=edge_strings.begin();sit!=edge_strings.end();sit++)
    {
	edge_string::iterator	esi;
	for(esi=(*sit)->begin();esi!=(*sit)->end();esi++)
	{
	    string_el_ptr sel( *esi );
	    edgel *edgel;
	    int			x,y;
		
	    edgel=sel->getEdgel();
	    x = (int)(edgel->getPosition()(0));
	    y = (int)(edgel->getPosition()(1));
	    if (getPixel(x,y)==0)
		setPixel( x,y, sel );
	}
    }
}


//
// WARNING: if you are going to add new operators here be Very careful
// 	    about which structure owns the pointed to props. usually
//	    these operations will only make sense if all vars are *not*
//	    owners.
//
void	property_image::operator+=( property_image& other)
{
    MMERROR( getWidth()==other.getWidth() && getHeight()==other.getHeight(),
             mimasexception, ,
             "Trying to add property images of different sizes, doing "
             "nothing." );
    
    int	x,y;
    for(y=0;y<getHeight();y++)
    {
	for(x=0;x<getWidth();++x)
	{
	    if (other.pixel(x,y) && !pixel(x,y))
	    {
         pixel(x,y) = other.pixel(x,y);
	    }
	}
    }
    
}
}

